from __future__ import annotations

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.animation import FuncAnimation
from matplotlib.lines import Line2D
from matplotlib.text import Annotation

from osaft.core.functions import pi
from osaft.plotting.datacontainers.wireframe_datacontainer import DeformedLine
from osaft.plotting.scattering.base_plotter import BaseScatteringPlotter


class WireframePlotter(BaseScatteringPlotter):
    """Plotter class for wireframe plot of the particle"""

    def __init__(self) -> None:
        """Constructor method"""
        super().__init__(is_scatterer=True)

    # -------------------------------------------------------------------------
    # API
    # -------------------------------------------------------------------------

    def plot(
        self,
        deformed_radii: list[DeformedLine],
        deformed_circles: list[DeformedLine],
        phase: float = 0,
        ax: None | plt.Axes = None,
    ) -> tuple[plt.Figure, plt.Axes]:
        """Plot wireframe

        :param deformed_radii: deformed line object, radii
        :param deformed_circles: deformed line object, circles
        :param phase: phase
        :param ax: matplotlib Axes object
        """
        # Create figure and axis
        fig, ax = self._create_figure(ax)

        # Loop through circles
        for circle in deformed_circles:
            self._plot_line(ax, circle, phase)

        # Loop through radii
        for radius in deformed_radii:
            self._plot_line(ax, radius, phase)

        # Finalize plot
        radius = self._get_radius(deformed_radii)
        self._set_ticks(ax, radius)
        self._add_labels(ax)

        return fig, ax

    def animate(
        self,
        frames: int,
        interval: float,
        deformed_radii: list[DeformedLine],
        deformed_circles: list[DeformedLine],
        ax: None | plt.Axes = None,
    ) -> FuncAnimation:
        """Animate wireframe

        :param frames: number of frames
        :param interval: interval between frames in ms
        :param deformed_radii: deformed line object, radii
        :param deformed_circles: deformed line object, circles
        :param ax: axes object
        """
        # Create figure and axis
        fig, ax = self._create_figure(ax)

        # Lists of lines
        list_plot_lines = []
        list_data_lines = deformed_circles + deformed_radii

        # Loop through circles and radii
        for circle in deformed_circles:
            _, line_def = self._plot_line(ax, circle, phase=0)
            list_plot_lines.append(line_def)
        for radius in deformed_radii:
            _, line_ref = self._plot_line(ax, radius, phase=0)
            list_plot_lines.append(line_ref)

        # Finalize plot
        radius = self._get_radius(deformed_radii)
        self._set_ticks(ax, radius)
        self._add_labels(ax)
        annotation = self._annotate_phase(ax, 0)

        anim = FuncAnimation(
            fig,
            self._animation_func,
            frames=frames,
            interval=interval,
            blit=False,
            fargs=(frames, list_data_lines, list_plot_lines, annotation),
        )

        return anim

    # -------------------------------------------------------------------------
    # Private Methods
    # -------------------------------------------------------------------------

    @staticmethod
    def _get_radius(deformed_radii: list[DeformedLine]) -> float:
        return np.max(deformed_radii[0].get_reference()[1])

    @staticmethod
    def _plot_line(
        ax: None | plt.Axes,
        deformed_line: DeformedLine,
        phase: float,
    ) -> tuple[Line2D, Line2D]:

        # Plot Reference Configuration
        x_ref, y_ref = deformed_line.get_reference()
        x_ref_sym = np.concatenate([x_ref, -x_ref])
        y_ref_sym = np.concatenate([y_ref, y_ref])
        (line_ref,) = ax.plot(
            y_ref_sym,
            x_ref_sym,
            alpha=0.5,
            color="grey",
            zorder=1,
        )

        # Deformed points
        x_def, y_def = deformed_line.get_deformed(phase)
        x_def_sym = np.concatenate([np.flip(x_def), -x_def])
        y_def_sym = np.concatenate([np.flip(y_def), y_def])

        # Plot Deformed Configuration
        (line_def,) = ax.plot(y_def_sym, x_def_sym, color="blue", lw=0.5)

        return line_ref, line_def

    def _animation_func(
        self,
        frame: int,
        n_frames: int,
        deformed_lines: list,
        plot_lines: list,
        annotation: Annotation,
    ):
        phase = frame / n_frames * 2 * pi
        for index, (data, line) in enumerate(zip(deformed_lines, plot_lines)):
            x, y = data.get_deformed(phase)
            x_def_sym = np.concatenate([np.flip(x), -x])
            y_def_sym = np.concatenate([np.flip(y), y])

            line.set_data(y_def_sym, x_def_sym)

        # Update phase annotation
        self._update_phase_annotation(annotation, phase)

        return plot_lines


if __name__ == "__main__":
    pass



from __future__ import annotations

import unittest
from collections.abc import Callable
from numbers import Number
from typing import Any

import numpy as np

from osaft import WaveType
from osaft.core.frequency import Frequency
from osaft.solutions.base_solution import BaseSolution
from osaft.tests.testing_parameters import (
    BaseChangingVariable,
    RandomParameters,
)


class NumericTestCase(unittest.TestCase):
    def assertAlmostEqual(
        self,
        val1: Number,
        val2: Number,
        threshold: None | float = 1e-12,
        print_results: None | bool = False,
        user_msg: None | str = None,
    ) -> None:
        """Asserting the relative difference of two values when num. rounding
        errors can occur

        Assertion is handled by :meth:`BaseTest._assertAlmostEqual()`

        If the values are complex, then the real and imaginary part will be
        tested separately. Also, if one of the test fails, automatically, a
        nice formatted output is generated as error message.

        :param val1: first value to be compared
        :param val2: second value to be compared
        :param threshold: threshold of relative difference, defaults to 1e-12
        :param print_results: if True the two values are printed to the console
            for easier debugging, defaults to False
        :param user_msg: custom message that is printed if the assertion fails.
        """
        if print_results:
            msg = f"\nval1     : {val1:+.8e}\n"
            msg += f"val2     : {val2:+.8e}\n"
            if not val1 == 0 and not val2 == 0:
                msg += f"rel diff.: {abs((val1 - val2) / val1):+.8e}"
            print(msg)

        self._assertAlmostEqual(val1.real, val2.real, threshold)
        if isinstance(val1, complex) or isinstance(val2, complex):
            self._assertAlmostEqual(val1.imag, val2.imag, threshold, user_msg)

    def _assertAlmostEqual(
        self,
        val1: float,
        val2: float,
        threshold: float,
        user_msg: None | str = None,
    ) -> None:

        msg = "\nAssertion failed\n"
        if user_msg:
            msg += f"\n{user_msg}\n"

        if val1 == 0 or val2 == 0:
            msg += "One of the values is Zero; the other not\n"
            msg += f"val1     : {val1:+.8e}\n"
            msg += f"val2     : {val2:+.8e}\n"
            test = abs(val1) < threshold and abs(val2) < threshold
            self.assertTrue(expr=test, msg=msg)
        else:
            diff = np.abs(val1 - val2)
            if np.abs(val1) > np.abs(val2):
                test = np.abs(diff / val2) < threshold
            else:
                test = np.abs(diff / val1) < threshold

            msg += "The rel. diff. is bigger than the threshold\n"
            msg += f"val1     : {val1:+.8e}\n"
            msg += f"val2     : {val2:+.8e}\n"
            msg += f"rel diff.: {abs(diff/val1):+.8e}\n"
            msg += f"threshold: {threshold:+.8e}"

            self.assertTrue(expr=test, msg=msg)


class BaseTestSolutions(NumericTestCase):
    def setUp(self):

        self.parameters = RandomParameters()

        # List of classes to be tests
        self.list_cls = []

        # Frequency Class
        self.frequency = Frequency(self.parameters.f)

        # Runs
        self._n_runs = self.parameters._wave_type.longest_list

    @property
    def n_runs(self) -> int:
        return self._n_runs

    @n_runs.setter
    def n_runs(self, val) -> None:
        if val < self.parameters._wave_type.longest_list:
            raise ValueError(
                "n_runs must be greater than the longest "
                "list of ChangingFromList._longest_list",
            )
        else:
            self._n_runs = val

    # -------------------------------------------------------------------------
    # Getters & Setters
    # -------------------------------------------------------------------------

    @property
    def R_0(self) -> float:
        return self.parameters.R_0

    @property
    def f(self) -> float:
        return self.parameters.f

    @property
    def c_f(self) -> float:
        return self.parameters.c_f

    @property
    def rho_f(self) -> float:
        return self.parameters.rho_f

    @property
    def c_s(self) -> float:
        return self.parameters.c_s

    @property
    def eta_s(self) -> float:
        return self.parameters.eta_s

    @property
    def zeta_s(self) -> float:
        return self.parameters.zeta_s

    @property
    def rho_s(self) -> float:
        return self.parameters.rho_s

    @property
    def eta_f(self) -> float:
        return self.parameters.eta_f

    @property
    def zeta_f(self) -> float:
        return self.parameters.zeta_f

    @property
    def eta_p(self) -> float:
        return self.parameters.eta_p

    @property
    def zeta_p(self) -> float:
        return self.parameters.zeta_p

    @property
    def lambda_M(self) -> float:
        return self.parameters.lambda_M

    @property
    def E_s(self) -> float:
        return self.parameters.E_s

    @property
    def nu_s(self) -> float:
        return self.parameters.nu_s

    @property
    def p_0(self) -> float:
        return self.parameters.p_0

    @property
    def position(self) -> float:
        return self.parameters.position

    @property
    def wave_type(self) -> WaveType:
        return self.parameters.wave_type

    @property
    def N_max(self):
        return self.parameters.N_max

    # -------------------------------------------------------------------------
    # Tests
    # -------------------------------------------------------------------------

    def test_name_attribute(self) -> None:
        if hasattr(self, "cls"):
            if isinstance(self.cls, BaseSolution):
                self.assertTrue(self.cls.name != "")

    def test_copy(self) -> None:
        if hasattr(self, "cls"):
            if isinstance(self.cls, BaseSolution):
                tmp = self.cls.copy()
                self.assertTrue(type(self.cls) == type(tmp))
                self.assertFalse(self.cls is tmp)

    # -------------------------------------------------------------------------
    # Generic test methods for properties and methods
    # -------------------------------------------------------------------------

    def _test_properties(
        self,
        list_of_properties: list[str],
        threshold: float = 1e-12,
    ) -> None:
        """Tests the properties inside `list_of_properties` of self & self.cls

        If a test fails the name of the failing property is printed.

        :param list_of_properties: list of properties to be tested
        :param threshold: comparing threshold for the whole list
        """

        for name in list_of_properties:
            with self.subTest(property=name):
                self.do_testing(
                    func_1=lambda: getattr(self, name),
                    func_2=lambda: getattr(self.cls, name),
                    threshold=threshold,
                )

    def _test_methods_n(
        self,
        dict_of_methods: dict[str, dict[tuple[Any]]],
        n_end: int = 5,
        threshold: float = 1e-12,
        zero: None | float = None,
    ) -> None:
        """Tests the methods inside `dict_of_methods` of self & self.cls

        All methods have as first argument `n`. The key of the dict is the name
        of the methods. The `value` of the dict are the optional additional
        arguments.  If `value` == None, then it is a method solely depending on
        `n`.

        If a test fails the name of the failing method is printed.

        :param dict_of_methods: dict of methods and additional arguments to be
            tested
        :param n_end: n in [0, 1, ..., n_end]
        :param threshold: comparing threshold for the whole dict
        :param threshold: value below which test values are assumed to be zero
        """

        for method, args in dict_of_methods.items():
            func_1 = getattr(self, method)
            func_2 = getattr(self.cls, method)
            for n in range(n_end, 0, -1):

                if args is not None:
                    if isinstance(args, tuple):
                        arg = (n, *args)
                    else:
                        arg = (n, args)
                else:
                    arg = n

                with self.subTest(method=method, arguments=arg):
                    self.do_testing(
                        func_1=func_1,
                        args_1=arg,
                        func_2=func_2,
                        args_2=arg,
                        threshold=threshold,
                        zero=zero,
                    )

    # -----------------------------------------------------------------------------
    # Tests
    # -----------------------------------------------------------------------------

    def do_testing(
        self,
        func_1: Callable,
        func_2: Callable,
        args_1: None | Any = None,
        args_2: None | Any = None,
        threshold: float = 1e-10,
        zero: None | float = None,
    ) -> None:
        """Compares the outputs of `func_1` against `func_2`.

        The variables in
        :attr:`BaseTest.test_variables` are changed and assigned one
        after the other to test all dependencies. This procedure is repeated
        :attr:`BaseTest.n_runs` times. Arguments to the functions can be
        passed with `args_1` and `args_2` to the respective functions.
        Testing is handled by :meth:`BaseTest._test_variables()` and changing
        of the values :meth:`BaseTest.change_and_assign_single_variable`

        In order to test properties of a class the following syntax is used:

        .. highlight:: python

        self.do_testing(lambda: self.cls.some_property, self.some_function)

        :param func_1: first function
        :param func_2: second function
        :param args_1: arguments for first function
        :param args_2: arguments for second functions
        :param threshold: threshold for failing the test, defaults to 1e-12
        :param zero: value below which test values are assumed to be zero
        """
        self.assign_parameters()
        self._test_variables(
            func_1,
            args_1,
            func_2,
            args_2,
            threshold,
            zero,
        )
        for _ in np.arange(self.n_runs):
            for var in self.parameters.list_parameters:
                if self._check_if_has_attr(var.name):
                    with self.subTest(msg=f"Changing {var.name}"):
                        self.change_and_assign_single_variable(var)
                        self._test_variables(
                            func_1,
                            args_1,
                            func_2,
                            args_2,
                            threshold,
                            zero,
                        )

    def _check_if_has_attr(self, name: str):
        """Check if any class in list_cls has the attribute `name`

        param name: name of the attribute
        """
        list_hasattr = [hasattr(cls, name) for cls in self.list_cls]
        return np.any(list_hasattr)

    def _test_variables(
        self,
        func_1: Callable,
        args_1: tuple,
        func_2: Callable,
        args_2: tuple,
        threshold: None | float = 1e-12,
        zero: None | float = None,
    ) -> None:
        """Checks if func_1(args) is almost equal to func_2 args

        Checks if func_1(args) is almost equal to func_2 args with tolerance
        `threshold`. Values below `zero` are assumed to be 0.

        :param func_1: first function
        :param args_1: arguments for the first function
        :param func_2: second function
        :param args_2: arguments for the second function
        :param threshold: tolerance
        "param zero: value below which test values are assumed to be zero
        """
        # check type of parameters1
        first = self._get_value(func_1, args_1)
        second = self._get_value(func_2, args_2)

        if zero is not None:
            first = 0 if abs(first) < zero else first
            second = 0 if abs(second) < zero else second

        self.assertAlmostEqual(first, second, threshold=threshold)

    @staticmethod
    def _get_value(func: Callable, args: tuple):
        """Get the value of the function `func` with arguments `arg`

        :param func: function
        :param args: arguments
        """
        if args is None:
            out = func()
        elif isinstance(args, tuple):
            out = func(*args)
        else:
            out = func(args)
        return out

    def assign_parameters(self) -> None:
        """Assigns the changed parameters to all instances

        This needs to be implemented in each derived object of `BaseTest`

        :raises NotImplementedError: Needs to be implemented in the
            derived object of BaseTest
        """
        for var in self.parameters.list_parameters:
            for cls in self.list_cls:
                if hasattr(cls, var.name):
                    setattr(cls, var.name, var.value)

    def change_and_assign_single_variable(
        self,
        var: BaseChangingVariable,
    ) -> None:
        """`var` is changed and its value is assigned so the attribute with
        the name `var.name`.

        The method `var.change` is called which randomly changes
        `var.value`. `var.value` is then assigned to the attribute of `self`
        with the name `var.name`.
        Assigning is handled by :meth:`BaseTest.assign_parameters()`

        :param var: Attribute name
        """
        var.change()
        self.assign_parameters()

    def assertAlmostEqual(
        self,
        val1: Number,
        val2: Number,
        threshold: None | float = 1e-12,
        print_results: None | bool = False,
    ) -> None:

        super().assertAlmostEqual(
            val1,
            val2,
            threshold,
            print_results,
            user_msg=f"\nseed number {self.parameters.seed}\n",
        )


if __name__ == "__main__":
    pass



import logging
from os import pardir
from os.path import abspath, dirname, join

# Debug file formatter
DEBUGFILE = "debug.log"
DEBUGFORMATTER = "%(filename)s:%(name)s:%(funcName)s:%(lineno)d: %(message)s"

# Log file and stream output formatter
INFOFILE = "info.log"
INFOFORMATTER = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

ROOT_DIR = dirname(abspath(__file__))
DEBUG_DIR = abspath(join(ROOT_DIR, pardir))
DEBUG_DIR += "/"

# define global logger
log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)

# define debug stream handler
db = logging.FileHandler(DEBUG_DIR + DEBUGFILE, mode="w")
db.setLevel(logging.DEBUG)
db.setFormatter(logging.Formatter(DEBUGFORMATTER))

# define info stream handler
info = logging.FileHandler(DEBUG_DIR + INFOFILE, mode="w")
info.setLevel(logging.INFO)
info.setFormatter(logging.Formatter(INFOFORMATTER))

# adds the handlers to the global variable: log
log.addHandler(db)
log.addHandler(info)



import os
import subprocess
import unittest

import numpy as np
from matplotlib import pyplot as plt


class BaseTestPlotting(unittest.TestCase):
    def setUp(self) -> None:
        self.create_output_folder("OutputPlotting")
        self.rng = np.random.default_rng(1)

    @property
    def output_folder_path(self) -> str:
        """Absolute folder path for all figure outputs"""
        return self._output_folder_path

    @property
    def root_folder(self) -> str:
        """Folder patch of the calling script"""
        return (
            subprocess.Popen(
                ["git", "rev-parse", "--show-toplevel"],
                stdout=subprocess.PIPE,
            )
            .communicate()[0]
            .rstrip()
            .decode("utf-8")
        )

    def _get_absolute_filename(self, name: str, file_type: str) -> str:
        """Return the absolute path of root_folder + name"""
        return os.path.join(self.output_folder_path, f"{name}.{file_type}")

    def create_output_folder(self, folder_name: str) -> None:
        """Creates a folder if non-existent relative to the root_folder"""
        abs_path = os.path.join(self.root_folder, folder_name)
        self._output_folder_path = abs_path
        if not os.path.isdir(abs_path):
            os.mkdir(abs_path)

    def save_fig(self, fig: plt.Figure, name: str, **kwargs) -> None:
        """Saves the figure

        :param fig: figures instance
        :param name: name of the file the figure is saved to
        :param **kwargs: passed to matplotlib.Figure.savefig
        """
        file_name = self._get_absolute_filename(name, "png")
        fig.savefig(file_name, **kwargs)
        plt.close(fig)


if __name__ == "__main__":
    pass



from __future__ import annotations

from abc import ABC

from osaft import log
from osaft.core.backgroundfields import WaveType
from osaft.core.frequency import Frequency
from osaft.core.functions import pi, sin
from osaft.core.geometries import Sphere
from osaft.core.helper import StringFormatter as SF
from osaft.core.variable import PassiveVariable
from osaft.solutions.basedoinikov1994.arf_limiting import BaseARFLimiting
from osaft.solutions.doinikov1994rigid.scattering import ScatteringField


class ARFLimiting(ScatteringField, BaseARFLimiting, ABC):
    """Child class for ARF according to Doinikov's theory
    (viscous fluid-rigid sphere; 1994)

    .. note::
        This class implements all attributes and methods to compute the ARF
        in the limiting cases introduces in the article by Doinikov
        :math:`|x| \\ll |x_v| \\ll 1` and :math:`|x| \\ll 1 \\ll |x_v|`.
        This class does not actually implement a method for the ARF. If you
        want to compute the ARF
        use :attr:`~osaft.solutions.doinikov1994rigid.ARF`.

    :param f: Frequency [Hz]
    :param R_0: Radius of the sphere [m]
    :param rho_s: Density of the sphere [kg/m^3]
    :param rho_f: Density of the fluid [kg/m^3]
    :param c_f: Speed of sound of the fluid [m/s]
    :param eta_f: shear viscosity [Pa s]
    :param zeta_f: bulk viscosity [Pa s]
    :param p_0: Pressure amplitude of the field [Pa]
    :param wave_type: Type of wave, travel(l)ing or standing
    :param position: Position in the standing wave field [rad]
    :param long_wavelength: using :math:`x \\ll 1`
    :param small_boundary_layer: :math:`x \\ll x_v \\ll 1`
    :param large_boundary_layer: :math`x \\ll 1 \\ll x_v`
    :param fastened_sphere: use theory of fastened sphere
    :param background_streaming: background streaming contribution
    :param N_max: Highest order mode
    """

    def __init__(
        self,
        f: Frequency | float | int,
        R_0: Sphere | float | int,
        rho_s: float,
        rho_f: float,
        c_f: float,
        eta_f: float,
        zeta_f: float,
        p_0: float,
        wave_type: WaveType,
        position: None | float,
        long_wavelength: bool,
        small_boundary_layer: bool,
        large_boundary_layer: bool,
        fastened_sphere: bool,
        background_streaming: bool,
        N_max: int,
    ) -> None:
        """Constructor method"""

        # init of parent class
        ScatteringField.__init__(
            self,
            f=f,
            R_0=R_0,
            rho_s=rho_s,
            rho_f=rho_f,
            c_f=c_f,
            eta_f=eta_f,
            zeta_f=zeta_f,
            p_0=p_0,
            wave_type=wave_type,
            position=position,
            N_max=N_max,
        )
        BaseARFLimiting.__init__(self)

        # independent variables
        self._fastened_sphere = PassiveVariable(
            fastened_sphere,
            "fastened sphere limit",
        )

        self._long_wavelength = PassiveVariable(
            long_wavelength,
            "long wavelength limit",
        )
        self._small_boundary_layer = PassiveVariable(
            small_boundary_layer,
            "small viscous boundary layer limit",
        )

        self._large_boundary_layer = PassiveVariable(
            large_boundary_layer,
            "large viscous boundary layer limit",
        )

        self._background_streaming = PassiveVariable(
            background_streaming,
            "background streaming contribution",
        )

        # logging
        log.debug(repr(self))
        log.info(str(self))

    def __repr__(self):
        return (
            f"Donikov1994Rigid.ARF(frequency={self.f}, radius={self.R_0}, "
            f"rho_s={self.rho_s}, rho_f={self.rho_f}, c_f={self.c_f}, "
            f"eta_f={self.eta_f}, zeta_f={self.zeta_f}, "
            f"p_0={self.p_0}, position={self.position}, {self.wave_type}, "
            f"small_viscous_boundary_layer="
            f"{self.small_boundary_layer}, "
            f"large_viscous_boundary_layer="
            f"{self.large_boundary_layer}, "
            f"small_particle_limit={self.long_wavelength}, "
            f"fastened_sphere= {self.fastened_sphere}, "
            f"N_max={self.N_max})"
        )

    def __str__(self):
        out = "Doinikovs's  (1994) model (viscous fluid-rigid sphere) for the"
        out += " ARF with the following properties: \n"
        out += "Limit Cases\n"
        out += SF.get_str_text(
            "Small particle limit",
            "small_particle_limit",
            self.long_wavelength,
            None,
        )
        out += SF.get_str_text(
            "Small delta",
            "small_viscous_boundary_layer",
            self.small_boundary_layer,
            None,
        )
        out += SF.get_str_text(
            "Fastened Sphere",
            "fastened_sphere",
            self.fastened_sphere,
            None,
        )
        out += "Backgroundfield\n"
        out += SF.get_str_text("Frequency", "f", self.f, "Hz")
        out += SF.get_str_text("Pressure", "p_0", self.p_0, "Pa")
        out += SF.get_str_text("Wavetype", "wave_type", self.wave_type, None)
        out += SF.get_str_text(
            "Position",
            "d",
            self.position,
            "rad",
        )
        out += SF.get_str_text(
            "Wavelength",
            "lambda",
            self.field.lambda_f,
            "m",
        )

        out += "Fluid\n"
        out += SF.get_str_text(
            "Density",
            "rho_f",
            self.rho_f,
            "kg/m^3",
        )
        out += SF.get_str_text(
            "Sound Speed",
            "c_0",
            self.c_f,
            "m/s",
        )
        out += SF.get_str_text(
            "Compressibility",
            "kappa_f",
            self.kappa_f,
            "1/Pa",
        )
        out += SF.get_str_text(
            "Shear viscosity",
            "eta_f",
            self.eta_f,
            "1/Pa",
        )
        out += SF.get_str_text(
            "Bulk viscosity",
            "zeta_f",
            self.zeta_f,
            "1/Pa",
        )

        out += "Particle\n"
        out += SF.get_str_text(
            "Radius",
            "R_0",
            self.R_0,
            "m",
        )
        out += SF.get_str_text(
            "Density",
            "rho_s",
            self.rho_s,
            "kg/m^3",
        )
        return out

    # -------------------------------------------------------------------------
    # Special Expressions for the ARF
    # -------------------------------------------------------------------------

    def _arf_fastened_small_delta(self) -> float:
        # eq. (7.1) for travelling wave and
        # eq. (6.14) w/ \\delta -> 0 for standing
        if self.wave_type == WaveType.TRAVELLING:
            out = 1 - self.norm_delta
            out *= 1.5 * self.norm_delta
        else:
            out = 5 / 6 + 0.75 * self.norm_delta
            out *= self._standing_wave_position_term()
        out *= pi * self.rho_f * self.field.abs_A_squared * self.x_0**3
        return out

    def _arf_fastened_large_delta(self) -> float:
        # eq. (7.5) for travelling wave and eq. (7.6) for standing
        if self.wave_type == WaveType.TRAVELLING:
            out = 1 - 1.2 / self.norm_delta

            out *= -1.5 * self.norm_delta
        else:
            out = 1 + 10 / 27 / self.norm_delta
            out *= 0.9 * self._standing_wave_position_term()
        out *= pi * self.rho_f * self.field.abs_A_squared * self.x_0**3
        out *= self.norm_delta
        return out

    def _arf_small_particle_large_delta(self) -> float:
        # eq. (6.18) for travelling wave and eq. (6.19) for standing
        if self.wave_type == WaveType.TRAVELLING:
            out = (1 - self.rho_t) / self.norm_delta
            out *= -22 / 15
        else:
            out = 11 * (1 - self.rho_t) / 5 / self.norm_delta
            out += 2 * self.rho_t - 1
            out *= self._standing_wave_position_term() / 3
        out *= pi * self.rho_s * self.field.abs_A_squared * self.x_0**3
        return out

    def _arf_small_particle_small_delta(self) -> float:
        # eq. (6.13) for travelling wave and eq. (6.14) for standing
        if self.wave_type == WaveType.TRAVELLING:
            out = (
                2
                + 12 * self.rho_t
                - 21 * self.rho_t**2
                + 7 * self.rho_t**3
            )
            out *= -self.norm_delta / (2 + self.rho_t)
            out += (1 - self.rho_t) ** 2
            out *= 6 * self.norm_delta
            out /= (2 + self.rho_t) ** 2
        else:
            out = 3 * (1 - self.rho_t) ** 2 * self.norm_delta
            out /= (2 + self.rho_t) ** 2
            out += (5 - 2 * self.rho_t) / (3 * (2 + self.rho_t))
            out *= self._standing_wave_position_term()
        out *= pi * self.rho_f * self.field.abs_A_squared * self.x_0**3
        return out

    def _standing_wave_position_term(self) -> float:
        return sin(2 * self.position)

    # -------------------------------------------------------------------------
    # Getters and Setters for independent variables
    # -------------------------------------------------------------------------

    @property
    def long_wavelength(self) -> bool:
        """Use limiting case for ARF calculation

        :getter: returns if a small particle limit is used
        :setter: automatically invokes
            :func:`src.core.variable.BaseVariable.notify`
        """
        return self._long_wavelength.value

    @long_wavelength.setter
    def long_wavelength(self, value):
        self._long_wavelength.value = value

    @property
    def fastened_sphere(self) -> bool:
        """Use limiting case of fastened sphere

        :getter: returns if fastened sphere limiting case is used
        :setter: automatically invokes
            :func:`src.core.variable.BaseVariable.notify`
        """
        return self._fastened_sphere.value

    @fastened_sphere.setter
    def fastened_sphere(self, value):
        self._fastened_sphere.value = value


if __name__ == "__main__":
    pass



from __future__ import annotations

from abc import ABC

import numpy as np
from matplotlib import pyplot as plt
from matplotlib.patches import Circle
from matplotlib.text import Annotation

from osaft.core.functions import pi, sqrt
from osaft.plotting.base_plotting import BasePlot

NDArray = np.ndarray
# handle depreacted warninig of matplotlib
plt.style.use("seaborn-v0_8-muted")


class BaseScatteringPlotter(BasePlot, ABC):
    """Base class for plotting first order solutions such as the acoustic
    scattering fields and particle vibrations.

    :param is_scatterer: if ``True``, plot of the scatterer,
        else plot of the fluid
    """

    def __init__(
        self,
        is_scatterer: bool,
    ) -> None:
        """Constructor method"""
        # Option if particle plot
        self.is_scatterer = is_scatterer

    # -------------------------------------------------------------------------
    # Figure handling
    # -------------------------------------------------------------------------

    def _create_figure(
        self,
        ax: plt.Axes,
    ) -> [None | plt.Figure, plt.Axes]:
        """
        Creates a matplotlib Figure instance and a matplotlib Axes instance and
        sets the aspect ratio to 1.
        If `ax` is passed no new instances are created and only the aspect
        ratio is set.

        :param ax: axes
        """
        fig, ax = super()._create_figure(ax)
        ax.set_aspect("equal", "box")
        ax.grid(False)
        return fig, ax

    # -------------------------------------------------------------------------
    # Post processing
    # -------------------------------------------------------------------------

    def _finish_plot(
        self,
        ax: plt.Axes,
        radius: float,
        xmax: None | float = None,
    ):
        """Post processes the plot
        Adds labels and ticks to the plot

        :param ax: axes
        :param radius: radius of the particle
        :param xmax: max value for plotting range
        """
        self._add_labels(ax)
        self._set_ticks(ax, radius)

    # ------------------------------------------------------------------------
    # Adding Artists and Annotations
    # -------------------------------------------------------------------------

    @staticmethod
    def _annotate_phase(ax: plt.Axes, phase: float) -> Annotation:
        """Annotates the phase to the axes

        Phase is annotated to the top right corner of the axes.

        :param ax: Axes object
        :param phase: phase
        """
        phase_by_pi = phase / pi
        txt = f"$\\varphi = {phase_by_pi:.2f}\\pi $"
        annotation = ax.annotate(
            txt,
            xy=(0.8, 0.94),
            xycoords="axes fraction",
            bbox=dict(
                boxstyle="square,pad=0.3",
                fc="white",
            ),
        )
        return annotation

    @staticmethod
    def _update_phase_annotation(
        annotation: Annotation,
        phase: float,
    ) -> Annotation:
        """Updates the phase

        :param annotation: Annotation object
        :param phase: phase
        """
        phase_by_pi = phase / pi
        txt = f"$\\varphi = {phase_by_pi:.2f}\\pi $"
        annotation.set_text(txt)
        return annotation

    @staticmethod
    def _add_circle(
        ax: plt.Axes,
        radius: float,
        linewidth: float = 0.5,
        fill: bool = False,
    ) -> None:
        """Draws the particle in the plot

        :param ax: axes
        :param radius: particle
        :param linewidth: linewidth
        :param fill: fill the circle
        """
        circle = Circle(
            (0, 0),
            radius=radius,
            linewidth=linewidth,
            fill=fill,
            color="white",
        )
        ax.add_artist(circle)

    # -------------------------------------------------------------------------
    # Labels and Ticks
    # -------------------------------------------------------------------------

    @staticmethod
    def _add_labels(ax: plt.Axes) -> None:
        """Adds axis labels to plot

        :param ax: axes
        """
        ax.set_xlabel("$z/R_0$ [-]")
        ax.set_ylabel("$x/R_0$ [-]")

    def _set_ticks(
        self,
        ax: plt.Axes,
        radius: float,
    ) -> None:
        """Sets ticks in plot

        :param ax: axes
        :param radius: particle radius
        """
        # Get ticks
        if self.is_scatterer:
            ticks, tick_labels = self._get_scatterer_ticks(radius)
        else:
            ticks, tick_labels = self._get_fluid_ticks(ax, radius)

        # Set ticks and labels
        ax.set_xticks(ticks, labels=tick_labels, minor=False)
        ax.set_yticks(ticks, labels=tick_labels, minor=False)

    @staticmethod
    def _get_fluid_ticks(
        ax: plt.Axes,
        radius: float,
    ) -> tuple[NDArray, NDArray]:
        """Sets ticks for a plot of the fluid

        :param radius: particle radius
        """
        # Compute ticks
        _, xmax = ax.get_xlim()
        right_radius = round(xmax / sqrt(2) / radius)
        range_radii = np.linspace(0, right_radius + 1, 3)
        pos_tick_labels = [round(label, 1) for label in range_radii]
        neg_tick_labels = [-label for label in pos_tick_labels if label != 0]
        tick_labels = np.array(pos_tick_labels + neg_tick_labels)
        ticks = tick_labels * radius
        return ticks, tick_labels

    @staticmethod
    def _get_scatterer_ticks(
        radius: float,
    ) -> tuple[NDArray, NDArray]:
        """Returns the ticks

        :param radius: particle radius
        """
        tick_labels = np.array([-1, -0.5, 0, 0.5, 1])
        ticks = tick_labels * radius
        return ticks, tick_labels

    @staticmethod
    def _set_labels(ax: plt.Axes):
        ax.set_xlabel("$z/R_0$")
        ax.set_ylabel("$x/R_0$")


if __name__ == "__main__":
    pass



from __future__ import annotations

import matplotlib.pyplot as plt
import numpy as np

from osaft.plotting.base_plotting import BasePlot
from osaft.plotting.datacontainers.arf_datacontainer import ARFData

NDArray = np.ndarray


class ARFPlotter(BasePlot):
    def __init__(self) -> None:
        self._abscissa = None
        self._attr_name = None

    def plot_solution(
        self,
        x_values: NDArray,
        data: ARFData,
        ax: plt.Axes,
        plot_method,
        **kwargs,
    ) -> (plt.Figure, plt.Axes):
        """plot_solution.

        :param x_values: abscissa data
        :param data: data to plot
        :param ax: axis object
        :param plot_method: plotting method to use
        :param kwargs: keyword arguments piped to :attr:`plot_method`
        """
        fig, ax = self._create_figure(ax)

        keys = kwargs.keys()
        if "ls" not in keys and "linestyle" not in keys:
            kwargs["linestyle"] = data.line_style

        plt.sca(ax)
        plot_method(
            x_values,
            data.plotting,
            label=data.name,
            **kwargs,
        )

        return fig, ax

    @staticmethod
    def add_legend(ax: plt.Axes) -> None:
        """Adds a legend to the given axis object

        :param ax:
        """
        ax.legend()

    @staticmethod
    def set_labels(
        ax: plt.Axes,
        attr_name: str,
        is_normalized: None | bool = False,
    ) -> None:
        """Sets the labels for both plotting axis

        :param ax: axis to work on
        :param attr_name: name of attribute on abscissa
        :param is_normalized: plotting of normalized solution
        """
        ax.set_xlabel(f"Attribute {attr_name}")

        if is_normalized:
            ax.set_ylabel("normalized ARF [-]")
        else:
            ax.set_ylabel(r"$F^{\mathrm{rad}}$ $\mathrm{[N]}$")


if __name__ == "__main__":
    pass



import unittest

from osaft.solutions.basedoinikov1994.base import BaseDoinikov1994
from osaft.tests.basetest_numeric import BaseTestSolutions


class BaseTestDoinikov1994(BaseTestSolutions):
    def setUp(self) -> None:

        super().setUp()
        self.cls = BaseDoinikov1994(
            self.f,
            self.R_0,
            self.rho_f,
            self.c_f,
            self.eta_f,
            self.zeta_f,
            self.p_0,
            self.wave_type,
            self.position,
        )

    @property
    def x_0(self) -> complex:
        return self.cls.field.k_f.real * self.R_0

    @property
    def x(self) -> complex:
        return self.cls.field.k_f * self.R_0

    @property
    def x_v(self) -> complex:
        return self.cls.field.k_v * self.R_0

    @property
    def norm_delta(self) -> complex:
        return self.cls.fluid.delta / self.R_0

    @property
    def rho_t(self):
        return self.rho_f / self.rho_s


class TestBaseDoinikov1994(BaseTestDoinikov1994):
    def test_properties(self) -> None:
        properties = ["x", "x_0", "x_v", "norm_delta"]
        self._test_properties(properties)


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import numpy as np

from osaft import log
from osaft.core.backgroundfields import WaveType
from osaft.core.frequency import Frequency
from osaft.core.geometries import Sphere
from osaft.core.warnings import EPS, raise_assumption_warning
from osaft.solutions.doinikov1994rigid.arf_general import ARFArbitraryRadius


class ARF(ARFArbitraryRadius):
    """ARF according to Doinikov's theory (viscous fluid-rigid sphere; 1994)

    There is a large number of options for the computation of the ARF in
    different limiting cases. For a more detailed explanation see
    :ref:`this example <sphx_glr_examples_tutorial_example_doinikov_1994.py>`.

    :param f: Frequency [Hz]
    :param R_0: Radius of the sphere [m]
    :param rho_s: Density of the sphere [kg/m^3]
    :param rho_f: Density of the fluid [kg/m^3]
    :param c_f: Speed of sound of the fluid [m/s]
    :param eta_f: shear viscosity [Pa s]
    :param zeta_f: bulk viscosity [Pa s]
    :param p_0: Pressure amplitude of the field [Pa]
    :param wave_type: Type of wave, traveling or standing
    :param position: Position within the standing wave field [m]
    :param long_wavelength: using :math:`x \\ll 1`
    :param small_boundary_layer: :math:`x \\ll x_v \\ll 1`
    :param large_boundary_layer: :math`x \\ll 1 \\ll x_v`
    :param fastened_sphere: use theory of fastened sphere
    :param background_streaming: background streaming contribution
    :param N_max: Highest order mode
    """

    def __init__(
        self,
        f: Frequency | float | int,
        R_0: Sphere | float | int,
        rho_s: float,
        rho_f: float,
        c_f: float,
        eta_f: float,
        zeta_f: float,
        p_0: float,
        wave_type: WaveType,
        position: None | float = None,
        long_wavelength: bool = False,
        small_boundary_layer: bool = False,
        large_boundary_layer: bool = False,
        fastened_sphere: bool = False,
        background_streaming: bool = True,
        N_max: int = 5,
    ) -> None:
        """Constructor method"""
        # Call to init method of parent class
        super().__init__(
            f,
            R_0,
            rho_s,
            rho_f,
            c_f,
            eta_f,
            zeta_f,
            p_0,
            wave_type,
            position,
            long_wavelength,
            small_boundary_layer,
            large_boundary_layer,
            fastened_sphere,
            background_streaming,
            N_max,
        )

    # -------------------------------------------------------------------------
    # API
    # -------------------------------------------------------------------------

    def compute_arf(self) -> float:
        """Acoustic radiation fore in [N]

        It logs the current values of :attr:`x` and :attr:`x_v`.

        :raises WrongWaveTypeError: if wrong :attr:`wave_type`
        :raises AssumptionWarning: if used solution might not be valid
        """
        # Checking wave_type
        self.check_wave_type()
        # Check Option combination
        self._check_option_combinations()

        log.info(
            "Computing the ARF with Doinikov1994Rigid.\n"
            f"x   = {self.x:+.4e} -> |x|   = {np.absolute(self.x):+.4e}\n"
            f"x_v = {self.x_v:+.4e} -> |x_v| = {np.absolute(self.x_v):+.4e}",
        )

        # Small Particle Solutions
        if not self.long_wavelength:
            return self._general_solution()

        # Checking x_v and x
        abs_x = np.abs(self.x)
        abs_x_v = np.abs(self.x_v)

        # Fastened Sphere Solution
        if self.fastened_sphere:
            if self.large_boundary_layer:
                out = self._arf_fastened_large_delta()
                test_assumption = abs_x < EPS * abs_x_v and EPS * abs_x_v < 1
            else:
                out = self._arf_fastened_small_delta()
                test_assumption = abs_x < EPS and 1 < EPS * abs_x_v
        else:
            # long wavelength and small boundary layer
            if self.small_boundary_layer:
                out = self._arf_small_particle_small_delta()
                test_assumption = abs_x < EPS and 1 < EPS * abs_x_v
            # long wavelength and large boundary layer
            elif self.large_boundary_layer:
                out = self._arf_small_particle_large_delta()
                test_assumption = abs_x < EPS * abs_x_v and EPS * abs_x_v < 1
            # long wavelength solution
            else:
                test_assumption = abs_x < EPS and abs_x < EPS * abs_x_v
                out = self._arf_small_particle()

        raise_assumption_warning(test_assumption)
        return out

    def _check_option_combinations(self):
        """Checks if the combination of options is correct"""
        allowed_combinations = [
            # General
            (False, False, False, False, False),
            (False, False, False, False, True),
            # Long wavelength, Arbitrary Boundary Layer
            (True, False, False, False, False),
            (True, False, False, False, True),
            # Long wavelength, Small Boundary Layer
            (True, True, False, False, True),
            (True, True, False, True, True),
            # Long wavelength, Large Boundary Layer
            (True, False, True, False, True),
            (True, False, True, True, True),
        ]
        combination = (
            self.long_wavelength,
            self.small_boundary_layer,
            self.large_boundary_layer,
            self.fastened_sphere,
            self.background_streaming,
        )
        if combination not in allowed_combinations:
            raise ValueError(
                f"The chosen combination {combination} for long_wavelength,"
                "small_boundary_layer, large_boundary_layer,"
                "fastened_sphere, background streaming is not allowed."
                f"Allowed options are: {allowed_combinations}",
            )


if __name__ == "__main__":
    pass



from __future__ import annotations

from osaft import log
from osaft.core.basecomposite import BaseFrequencyComposite
from osaft.core.frequency import Frequency
from osaft.core.functions import sqrt
from osaft.core.variable import ActiveVariable, PassiveVariable


class RigidSolid(BaseFrequencyComposite):
    """RigidSolid class

    :param frequency: excitation frequency in [Hz]
    :param rho: density in [kg/m^3]
    """

    def __init__(
        self,
        frequency: int | float | Frequency,
        rho: float,
    ) -> None:
        """Constructor method"""

        # Calling parent class
        super().__init__(frequency)

        # Independent variables
        self._rho_s = PassiveVariable(rho, "density rho")

        if type(self) is RigidSolid:
            log.debug(f"Creating {self}")

    def __repr__(self) -> str:
        return f"RigidSolid(f={self.f}, rho={self.rho_s})"

    # -------------------------------------------------------------------------
    # Setters and Getters for Independent Variables
    # -------------------------------------------------------------------------

    @property
    def rho_s(self) -> float:
        """Returns the density  :math:`\\rho_{s}`.

        :getter: returns the value for the density
        :setter: automatically invokes
            :meth:`osaft.core.variable.BaseVariable.notify`
        """
        return self._rho_s.value

    @rho_s.setter
    def rho_s(self, value: float) -> None:
        self._rho_s.value = value


class ElasticSolid(RigidSolid):
    """ElasticSolid class

    :param frequency: excitation frequency in [Hz]
    :param E: Young's modulus in [Pa]
    :param nu: Poisson's ratio in [-]
    :param rho: density in [km m^-3]
    """

    def __init__(self, frequency, E, nu, rho) -> None:
        """Constructor method"""

        super().__init__(frequency, rho)

        # Independent variables
        self._E_s = PassiveVariable(E, "Young's modulus E")
        self._nu_s = PassiveVariable(nu, "Poisson's ratio nu")

        # Dependent variables
        self._c_l = ActiveVariable(self._compute_c_l, "primary wave speed c_l")
        self._k_l = ActiveVariable(
            self._compute_k_l,
            "wavenumber of primary wave k_l",
        )

        self._c_t = ActiveVariable(
            self._compute_c_t,
            "secondary wave speed c_t",
        )
        self._k_t = ActiveVariable(
            self._compute_k_t,
            "wavenumber of secondary wave k_t",
        )

        self._G = ActiveVariable(self._compute_G, "shear modulus G")
        self._lambda = ActiveVariable(
            self._compute_lambda,
            "second Lame parameter lambda",
        )

        self._B_s = ActiveVariable(self._compute_B_s, "bulk modulus B_s")
        self._kappa_s = ActiveVariable(
            self._compute_kappa_s,
            "compressibility kappa_s",
        )

        # Dependencies
        self._G.is_computed_by(self._E_s, self._nu_s)
        self._lambda.is_computed_by(self._E_s, self._nu_s)

        self._c_l.is_computed_by(self._E_s, self._nu_s, self._rho_s)
        self._k_l.is_computed_by(self._c_l, self.frequency._omega)

        self._c_t.is_computed_by(self._E_s, self._nu_s, self._rho_s)
        self._k_t.is_computed_by(self._c_t, self.frequency._omega)

        self._B_s.is_computed_by(self._E_s, self._nu_s)
        self._kappa_s.is_computed_by(self._B_s)

        # no if clause needed because last in inheritance diagram
        log.debug(f"Creating {self}")

    def __repr__(self) -> str:
        return (
            f"ElasticSolid(f={self.f}, rho={self.rho_s}, "
            f"E={self.E_s}, nu={self.nu_s})"
        )

    # -------------------------------------------------------------------------
    # Setters and Getters for Independent Variables
    # -------------------------------------------------------------------------

    @property
    def E_s(self) -> float:
        """Returns the Young's modulus :math:`E_{s}` [Pa].

        :getter: returns the value for the Young's modulus
        :setter: automatically invokes
            :meth:`osaft.core.variable.BaseVariable.notify`
        """
        return self._E_s.value

    @E_s.setter
    def E_s(self, value: float) -> None:
        self._E_s.value = value

    @property
    def nu_s(self) -> float:
        """Returns the Poisson's ratio :math:`\\nu_{s}`.

        :getter: returns the value for the Poisson's ratio
        :setter: automatically invokes
            :meth:`osaft.core.variable.BaseVariable.notify`
        """
        return self._nu_s.value

    @nu_s.setter
    def nu_s(self, value: float) -> None:
        if not (0 <= value <= 0.5):
            raise ValueError(
                "The Poisson's ratio `nu` needs to be in the "
                "range 0 <= nu <= 0.5",
            )
        self._nu_s.value = value

    # -------------------------------------------------------------------------
    # Getters for Dependent Variables
    # -------------------------------------------------------------------------

    @property
    def G(self) -> float:
        """Returns the shear modulus :math:`G`."""
        return self._G.value

    @property
    def B_s(self) -> float:
        """Returns the bulk modulus :math:`B_{s}` [Pa]"""
        return self._B_s.value

    @property
    def kappa_s(self) -> float:
        """Returns the compressibility :math:`\\kappa_{s}` [1/Pa]"""
        return self._kappa_s.value

    @property
    def c_l(self) -> float:
        """Returns the longitudinal wave speed :math:`c_l` [m/s]"""
        return self._c_l.value

    @property
    def c_t(self) -> float:
        """Return the transverse wave speed :math:`c_t` [m/s]"""
        return self._c_t.value

    @property
    def k_l(self) -> float:
        """Returns the longitudinal wave number :math:`k_l` [1/m]"""
        return self._k_l.value

    @property
    def k_t(self) -> float:
        """Returns the transverse wave number :math:`k_t` [1/m]"""
        return self._k_t.value

    @property
    def lame_1(self) -> float:
        """Returns the first Lamé parameter :math:`\\lambda` [Pa]"""
        return self._lambda.value

    @property
    def lame_2(self) -> float:
        """Returns the second Lamé parameter :math:`\\mu`"""
        return self.G

    # -------------------------------------------------------------------------
    # Dependent Variables Methods
    # -------------------------------------------------------------------------

    def _compute_G(self) -> float:
        return self.G_from_E_nu(self.E_s, self.nu_s)

    def _compute_lambda(self) -> float:
        return self.lambda_from_E_nu(self.E_s, self.nu_s)

    def _compute_B_s(self) -> float:
        return self.E_s / (1 - 2 * self.nu_s) / 3

    def _compute_kappa_s(self) -> float:
        return 1 / self.B_s

    def _compute_c_l(self) -> float:
        return self.c1_from_E_nu(self.E_s, self.nu_s, self.rho_s)

    def _compute_c_t(self) -> float:
        return self.c2_from_E_nu(self.E_s, self.nu_s, self.rho_s)

    def _compute_k_l(self) -> float:
        return self.omega / self.c_l

    def _compute_k_t(self) -> float:
        return self.omega / self.c_t

    # -------------------------------------------------------------------------
    # Conversion Methods
    # -------------------------------------------------------------------------

    @staticmethod
    def E_from_wave_speed(
        c_1: float,
        c_2: float,
        rho: float,
    ) -> float:
        """Computes the Young's modulus :math:`E` from the primary and the
        secondary wave speed :math:`c_1`, :math:`c_2` and the density
        :math:`\\rho`.

        :param c_1: primary wave speed [m/s]
        :param c_2: secondary wave speed [m/s]
        :param rho: density [kg/m^3]
        """
        return (
            rho
            * c_2**2
            * (3 * c_1**2 - 4 * c_2**2)
            / (c_1**2 - c_2**2)
        )

    @staticmethod
    def nu_from_wave_speed(
        c_1: float,
        c_2: float,
    ) -> float:
        """Computes the Poisson's ratio :math:`\\nu` from the primary and the
        secondary wave speed :math:`c_1`, :math:`c_2`, and the density
        :math:`\\rho`.

        :param c_1: primary wave speed [m/s]
        :param c_2: secondary wave speed [m/s]
        """
        return (1 - 2 * (c_2 / c_1) ** 2) / (2 - 2 * (c_2 / c_1) ** 2)

    @staticmethod
    def lambda_from_wave_speed(
        c_1: float,
        c_2: float,
        rho: float,
    ) -> float:
        """Computes the Lamé first parameter :math:`\\lambda` from the
        primary and the secondary wave speed :math:`c_1`, :math:`c_2` and the
        density :math:`\\rho`.

        :param c_1: primary wave speed [m/s]
        :param c_2: secondary wave speed [m/s]
        :param rho: density [kg/m^3]
        """
        G = c_2**2 * rho
        return c_1**2 * rho - 2 * G

    @staticmethod
    def G_from_wave_speed(
        c_2: float,
        rho: float,
    ) -> float:
        """Computes the second Lamé parameter :math:`G` from the secondary
        wave speed :math:`c_2` and the density :math:`\\rho`.

        :param c_2: secondary wave speed [m/s]
        :param rho: density [kg/m^3]
        """
        return c_2**2 * rho

    @staticmethod
    def c1_from_E_nu(
        E: float,
        nu: float,
        rho: float,
    ) -> float:
        """Computes the primary wave speed :math:`c_1` from the Young's modulus
        :math:`E` and the Poisson's ratio :math:`\\nu`, and the density
        :math:`\\rho`.

        :param E: Young;s modulus [Pa]
        :param nu: Poisson's ratio [-]
        :param rho: density [kg/m^3]
        """
        return sqrt(E * (1 - nu) / (rho * (1 + nu) * (1 - 2 * nu)))

    @staticmethod
    def c2_from_E_nu(
        E: float,
        nu: float,
        rho: float,
    ) -> float:
        """Computes the secondary wave speed :math:`c_2` from the Young's
        modulus :math:`E` and the Poisson's ratio :math:`\\nu`, and the density
        :math:`\\rho`.

        :param E: Young;s modulus [Pa]
        :param nu: Poisson's ratio [-]
        :param rho: density [kg/m^3]
        """
        return sqrt(E / (2 * (1 + nu) * rho))

    @staticmethod
    def c1_from_Lame(
        lam: float,
        G: float,
        rho: float,
    ) -> float:
        """Computes the primary wave speed :math:`c_1` from the Lamé parameters
        :math:`\\lambda`, :math:`G`, and the density
        :math:`\\rho`.

        :param lam: first Lamé parameter [Pa]
        :param G: second Lamé parameter [Pa]
        :param rho: density [kg/m^3]
        """
        return sqrt((lam + 2 * G) / rho)

    @staticmethod
    def c2_from_Lame(
        G: float,
        rho: float,
    ) -> float:
        """Computes the secondary wave speed :math:`c_2` from the Lamé
        parameters :math:`\\lambda`, :math:`G`, and the density :math:`\\rho`.

        :param G: second Lamé parameter [Pa]
        :param rho: density [kg/m^3]
        """
        return sqrt(G / rho)

    @staticmethod
    def E_from_Lame(
        lam: float,
        G: float,
    ) -> float:
        """Computes the Young's modulus :math:`E` from the Lamé parameters
        :math:`\\lambda`, :math:`G`

        :param lam: first Lamé parameter [Pa]
        :param G: second Lamé parameter [Pa]
        """
        return G * (3 * lam + 2 * G) / (lam + G)

    @staticmethod
    def nu_from_Lame(
        lam: float,
        G: float,
    ) -> float:
        """Computes the the Poisson's ratio :math:`\\nu` from the Lamé
        parameters :math:`\\lambda`, :math:`G`.

        :param lam: first Lamé parameter [Pa]
        :param G: second Lamé parameter [Pa]
        """
        return lam / (2 * (lam + G))

    @classmethod
    def lambda_from_E_nu(cls, E: float, nu: float) -> float:
        """Computes the first Lamé parameters :math:`\\lambda` from the Young's
        modulus :math:`E` and the Poisson's ratio :math:`\\nu`.

        :param E: Young;s modulus [Pa]
        :param nu: Poisson's ratio [-]
        """
        G = cls.G_from_E_nu(E, nu)
        return 2 * G * nu / (1 - 2 * nu)

    @staticmethod
    def G_from_E_nu(
        E: float,
        nu: float,
    ) -> float:
        """Computes the second Lamé parameters :math:`G` from the Young's
        modulus :math:`E` and the Poisson's ratio :math:`\\nu`.

        :param E: Young;s modulus [Pa]
        :param nu: Poisson's ratio [-]
        """
        return E / (2 * (1 + nu))


if __name__ == "__main__":
    pass



import codecs
import os.path
import re

from setuptools import find_packages, setup

with open("requirements.txt") as f:
    install_requires = f.read().splitlines()

with open("requirements_dev.txt") as f:
    dev_requires = f.read().splitlines()

with open("docs/requirements.txt") as f:
    docs_requires = f.read().splitlines()

with open("README.md") as f:
    long_description = f.read()


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        match = re.search(r"## (\d{1,2}\.\d{1,2}\.\d{1,2})", line)
        if match is not None:
            match = match.group(1)
            return str(match)
    else:
        raise RuntimeError("Unable to find version string.")


setup(
    name="osaft",
    version=get_version("CHANGELOG.md"),
    packages=find_packages(),
    description="An Open-Source Python Library For Acoustofluidics",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="osaftpy developer team",
    author_email="osaftpy@gmail.com",
    url="https://gitlab.com/acoustofluidics/osaft",
    project_urls={
        "Documentation": "https://osaft.readthedocs.io/en/stable/",
        "Tracker": "https://gitlab.com/acoustofluidics/osaft/-/issues",
    },
    install_requires=install_requires,
    extras_require={
        "dev": dev_requires,
        "docs": docs_requires,
    },
    test_suite="tests",
    keywords="msaf arf scattering acoustofluidic acoustic-radiation-force",
    license="LPGL v3",
    python_requires=">=3.9",
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Physics",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: GNU Lesser General Public License v3 "
        "(LGPLv3)",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)



from __future__ import annotations

from osaft.core.backgroundfields import BackgroundField, WaveType
from osaft.core.basecomposite import BaseSphereFrequencyComposite
from osaft.core.fluids import InviscidFluid
from osaft.core.frequency import Frequency
from osaft.core.geometries import Sphere
from osaft.core.solids import RigidSolid
from osaft.core.variable import ActiveVariable
from osaft.solutions.base_solution import BaseSolution


class BaseKing(BaseSphereFrequencyComposite, BaseSolution):
    """Base class for King (1934)

    :param f: Frequency [Hz]
    :param R_0: Radius of the sphere [m]
    :param rho_s: Density of the fluid-like sphere [kg/m^3]
    :param rho_f: Density of the fluid [kg/m^3]
    :param c_f: Speed of sound of the fluid [m/s]
    :param p_0: Pressure amplitude of the field [Pa]
    :param wave_type: Type of incident wave (traveling/standing)
    :param position: Position in the standing wave field [rad]
    """

    # Supported wave_type for the class
    supported_wavetypes = [WaveType.STANDING, WaveType.TRAVELLING]

    def __init__(
        self,
        f: Frequency | float | int,
        R_0: Sphere | float | int,
        rho_s: float,
        rho_f: float,
        c_f: float,
        p_0: float,
        wave_type: WaveType,
        position: None | float,
    ) -> None:
        """Constructor method"""

        # init of parent class
        BaseSphereFrequencyComposite.__init__(self, f, R_0)
        BaseSolution.__init__(self, "King1934")

        # Initialize Components
        self.scatterer = RigidSolid(self.frequency, rho_s)
        self.fluid = InviscidFluid(self.frequency, rho_f, c_f)
        self.field = BackgroundField(self.fluid, p_0, wave_type, position)

        self._alpha = ActiveVariable(
            self._compute_alpha,
            "wave number times radius of sphere",
        )
        self._rho_tilde = ActiveVariable(
            self._compute_rho_tilde,
            "rho_s over rho_f",
        )

        # Dependencies
        self._alpha.is_computed_by(self.fluid._k_f, self.sphere._R_0)
        self._rho_tilde.is_computed_by(
            self.fluid._rho_f,
            self.scatterer._rho_s,
        )

    # -------------------------------------------------------------------------
    # Getters for Dependent Variables
    # -------------------------------------------------------------------------

    @property
    def alpha(self):
        """Wave number times radius of the particle (:math:`kR`) [-]"""
        return self._alpha.value

    @property
    def rho_tilde(self) -> float:
        """Density ratio :attr:`~.rho_s`/ :attr:`~.rho_f` [-]"""
        return self._rho_tilde.value

    # -------------------------------------------------------------------------
    # Wrappers for Independent Field Attributes
    # -------------------------------------------------------------------------

    @property
    def position(self) -> float:
        """Wraps to
        :attr:`osaft.core.backgroundfields.BackgroundField.position`
        """
        return self.field.position

    @position.setter
    def position(self, value: float) -> None:
        self.field.position = value

    @property
    def p_0(self) -> float:
        """Wraps to
        :attr:`osaft.core.backgroundfields.BackgroundField.p_0`
        """
        return self.field.p_0

    @p_0.setter
    def p_0(self, value: float) -> None:
        self.field.p_0 = value

    @property
    def wave_type(self) -> WaveType:
        """Wraps to
        :attr:`osaft.core.backgroundfields.BackgroundField.wave_type`
        """
        return self.field.wave_type

    @wave_type.setter
    def wave_type(self, value: WaveType) -> None:
        self.field.wave_type = value

    def A_in(self, n: int) -> complex:
        """Wraps to
        :attr:`osaft.core.backgroundfields.BackgroundField.A_in`
        """
        return self.field.A_in(n)

    # -------------------------------------------------------------------------
    # Wrappers for Independent Scatterer Attributes
    # -------------------------------------------------------------------------

    @property
    def rho_s(self) -> float:
        """Wraps to :attr:`osaft.core.solids.RigidSolid.rho_s`"""
        return self.scatterer.rho_s

    @rho_s.setter
    def rho_s(self, value: float) -> None:
        self.scatterer.rho_s = value

    # -------------------------------------------------------------------------
    # Wrappers for Independent Fluid Attributes
    # -------------------------------------------------------------------------

    @property
    def rho_f(self) -> float:
        """Wraps to :attr:`osaft.core.fluids.InviscidFluid.rho_f`"""
        return self.fluid.rho_f

    @rho_f.setter
    def rho_f(self, value: float) -> None:
        self.fluid.rho_f = value

    @property
    def c_f(self) -> float:
        """Wraps to :attr:`osaft.core.fluids.InviscidFluid.c_f`"""
        return self.fluid.c_f

    @c_f.setter
    def c_f(self, value: float) -> None:
        self.fluid.c_f = value

    # -------------------------------------------------------------------------
    # Wrappers for Dependent Fluid Attributes
    # -------------------------------------------------------------------------

    @property
    def kappa_f(self) -> float:
        """Wraps to :attr:`osaft.core.fluids.InviscidFluid.kappa_f`"""
        return self.fluid.kappa_f

    @property
    def k_f(self) -> float:
        """Wraps to :attr:`osaft.core.fluids.InviscidFluid.k_f`"""
        return self.field.k_f

    # -------------------------------------------------------------------------
    # Dependent Variables Methods
    # -------------------------------------------------------------------------

    def _compute_alpha(self) -> float:
        return self.k_f * self.R_0

    def _compute_rho_tilde(self) -> float:
        return self.rho_s / self.rho_f


if __name__ == "__main__":
    pass



import unittest

import osaft
from osaft import WaveType
from osaft.core.functions import BesselFunctions as Bf
from osaft.tests.basetest_arf import HelperCompareARF
from osaft.tests.basetest_numeric import BaseTestSolutions
from osaft.tests.solution_factory import SolutionFactory


class TestARF(BaseTestSolutions):
    def setUp(self) -> None:
        super().setUp()

        self.cls = SolutionFactory().hasegawa_1969_arf()

        self.list_cls = [self.cls]

    # -------------------------------------------------------------------------
    # Methods
    # -------------------------------------------------------------------------

    def U_n(self, n):
        cn = self.cls.c_n(n)
        x = self.cls.x_f
        j = Bf.besselj(n, x)
        n = Bf.bessely(n, x)
        return (1 + cn.real) * j + cn.imag * n

    def V_n(self, n):
        cn = self.cls.c_n(n)
        x = self.cls.x_f
        j = Bf.besselj(n, x)
        n = Bf.bessely(n, x)
        return cn.imag * j - cn.real * n

    def U_n_d1(self, n):
        cn = self.cls.c_n(n)
        x = self.cls.x_f
        dj = Bf.d1_besselj(n, x)
        dn = Bf.d1_bessely(n, x)
        return (1 + cn.real) * dj + cn.imag * dn

    def V_n_d1(self, n):
        cn = self.cls.c_n(n)
        x = self.cls.x_f
        dj = Bf.d1_besselj(n, x)
        dn = Bf.d1_bessely(n, x)
        return cn.imag * dj - cn.real * dn

    def _compute_arf(self):
        s = 0
        for n in range(self.cls.N_max):
            s1 = (
                -self.cls.x_f**2
                * (n + 1)
                * (
                    self.U_n_d1(n) * self.V_n_d1(n + 1)
                    - self.V_n_d1(n) * self.U_n_d1(n + 1)
                )
            )
            s2 = (
                n
                * (n + 1)
                * (n + 2)
                * (
                    self.U_n(n) * self.V_n(n + 1)
                    - self.V_n(n) * self.U_n(n + 1)
                )
            )
            s3 = (
                self.cls.x_f
                * n
                * (n + 1)
                * (
                    self.U_n(n) * self.V_n_d1(n + 1)
                    - self.V_n(n) * self.U_n_d1(n + 1)
                )
            )
            s4 = (
                -self.cls.x_f
                * (n + 1)
                * (n + 2)
                * (
                    self.U_n_d1(n) * self.V_n(n + 1)
                    - self.V_n_d1(n) * self.U_n(n + 1)
                )
            )
            s5 = (
                -self.cls.x_f**2
                * (n + 1)
                * (
                    self.U_n(n) * self.V_n(n + 1)
                    - self.V_n(n) * self.U_n(n + 1)
                )
            )
            s += s1 + s2 + s3 + s4 + s5

        s *= 2 * osaft.pi * self.rho_f * abs(self.cls.field.A) ** 2
        return s

    def _compute_arf_standing(self):
        s = 0
        for n in range(self.cls.N_max):
            s1 = (
                -self.cls.x_f**2
                * (n + 1)
                * (-1) ** (n + 1)
                * (
                    self.U_n_d1(n) * self.U_n_d1(n + 1)
                    + self.V_n_d1(n) * self.V_n_d1(n + 1)
                )
            )
            s2 = (
                n
                * (n + 1)
                * (n + 2)
                * (-1) ** (n + 1)
                * (
                    self.U_n(n) * self.U_n(n + 1)
                    + self.V_n(n) * self.V_n(n + 1)
                )
            )
            s3 = (
                self.cls.x_f
                * n
                * (n + 1)
                * (-1) ** (n + 1)
                * (
                    self.U_n(n) * self.U_n_d1(n + 1)
                    + self.V_n(n) * self.V_n_d1(n + 1)
                )
            )
            s4 = (
                -self.cls.x_f
                * (n + 1)
                * (n + 2)
                * (-1) ** (n + 1)
                * (
                    self.U_n_d1(n) * self.U_n(n + 1)
                    + self.V_n_d1(n) * self.V_n(n + 1)
                )
            )
            s5 = (
                -self.cls.x_f**2
                * (n + 1)
                * (-1) ** (n + 1)
                * (
                    self.U_n(n) * self.U_n(n + 1)
                    + self.V_n(n) * self.V_n(n + 1)
                )
            )
            s += s1 + s2 + s3 + s4 + s5

        s *= (
            1
            * osaft.pi
            * self.rho_f
            * self.cls.field.A**2
            * osaft.core.functions.sin(
                2 * self.cls.position / self.cls.k_f.real * self.cls.k_f,
            )
        )
        return -s.real

    def compute_arf(self):
        self.cls.check_wave_type()
        if self.cls.wave_type == WaveType.STANDING:
            return self._compute_arf_standing()
        else:
            return self._compute_arf()

    def _compute_arf_from_Yp(self) -> float:
        """
        Computes general analytical solution for the
        acoustic radiation force function.
        Replace in compute_arf to use for simple plotting.
        Used to recreate figures that are in the paper.
        (Eq. 29)
        """
        out = 4 * [0]

        for n in range(self.N_max):
            # (29, first row)
            s1 = self.V_n_d1(n) * self.U_n_d1(n + 1)
            s1 -= self.U_n_d1(n) * self.V_n_d1(n + 1)
            s1 *= n + 1
            out[0] += s1

            # (29, second row)
            s2 = self.V_n(n) * self.U_n(n + 1)
            s2 -= self.U_n(n) * self.V_n(n + 1)
            s2 *= n * (n + 1) * (n + 2)
            out[1] += s2

            # (29, third and fourth row)
            s3 = self.U_n(n) * self.V_n_d1(n + 1)
            s3 -= self.V_n(n) * self.U_n_d1(n + 1)
            s3 *= n * (n + 1)
            s4 = self.U_n_d1(n) * self.V_n(n + 1)
            s4 -= self.V_n_d1(n) * self.U_n(n + 1)
            s4 *= (n + 1) * (n + 2)
            out[2] += s3
            out[2] -= s4
            # (29, last row)
            s5 = self.V_n(n) * self.U_n(n + 1)
            s5 -= self.U_n(n) * self.V_n(n + 1)
            s5 *= n + 1
            out[3] += s5

        out[0] *= self.cls.x_f**2
        out[1] *= -1
        out[2] *= self.cls.x_f
        out[3] *= self.cls.x_f**2

        out = 4 / (self.cls.x_f**2) * sum(out)
        E = 0.5 * self.cls.rho_f * self.cls.k_f**2 * self.cls.field.A**2
        out *= osaft.pi * self.cls.R_0**2 * E
        return -out

    def _compute_arf_from_Yp_standing(self) -> float:
        """
        Computes general analytical solution for the
        acoustic radiation force function, standing wave case.
        Replace in compute_arf to use for simple plotting.
        Used to recreate figures that are in the paper.
        (Eq. 19 in 1979 paper)
        """
        out = 0
        for n in range(self.N_max):
            out += (
                2
                * (n + 1)
                * (-1) ** (n + 1)
                * (
                    self.cls.c_n(n).imag * (1 + 2 * self.cls.c_n(n + 1).real)
                    - self.cls.c_n(n + 1).imag * (1 + 2 * self.cls.c_n(n).real)
                )
                / self.cls.x_f**2
            )

        # (Eq. 29)
        E = 0.5 * self.cls.rho_f * self.cls.k_f**2 * self.cls.field.A**2
        sin = osaft.core.functions.sin(
            2 * self.position / self.cls.k_f.real * self.cls.k_f,
        )
        out *= osaft.pi * self.cls.R_0**2 * E * sin
        return out.real

    def compute_arf_from_Yp(self) -> float:
        """
        Acoustic radiation force function [-]

        Computes the radiation force function based on (29, Hasegawa1969) and
        (19, Hasegawa1979).
        To check if the functions are implemented correctly, as you can
        compare to the figures in the paper.

        :raises WrongWaveTypeError: if wrong :attr:`wave_type`
        """

        # Checking wave_type
        self.cls.check_wave_type()

        if self.wave_type == WaveType.STANDING:
            return self._compute_arf_from_Yp_standing()
        else:
            return self._compute_arf_from_Yp()

    # -------------------------------------------------------------------------
    # Test
    # -------------------------------------------------------------------------

    def test_U_n(self):
        for n in range(self.n_runs):
            self.do_testing(self.cls.U_n, self.U_n, n, n)

    def test_V_n(self):
        for n in range(self.n_runs):
            self.do_testing(self.cls.V_n, self.V_n, n, n)

    def test_U_n_d1(self):
        for n in range(self.n_runs):
            self.do_testing(self.cls.d_U_n, self.U_n_d1, n, n)

    def test_V_n_d1(self):
        for n in range(self.n_runs):
            self.do_testing(self.cls.d_V_n, self.V_n_d1, n, n)

    def test_arf(self):
        self.do_testing(
            self.compute_arf,
            self.cls.compute_arf,
        )

    def test_arf_from_Yp(self):
        self.do_testing(
            self.compute_arf_from_Yp,
            self.cls.compute_arf,
        )


class TestCompareToYosioka(BaseTestSolutions, HelperCompareARF):
    def setUp(self) -> None:

        BaseTestSolutions.setUp(self)

        self.arf_compare_threshold = 5e-3

        self.cls = SolutionFactory().hasegawa_1969_arf()
        self.compare_cls = SolutionFactory().yosioka_1955_arf()

        self.parameters.list_parameters.remove(self.parameters._rho_s)
        self.parameters.list_parameters.remove(self.parameters._E_s)
        self.parameters.list_parameters.remove(self.parameters._nu_s)
        self.parameters.list_parameters.remove(self.parameters._c_s)

        self.parameters._R_0.low = 1e-6
        self.parameters._R_0.high = 5e-6

        self.cls.E_s = (
            3 * (1 - 2 * self.nu_s) / self.compare_cls.scatterer.kappa_f
        )

        self.list_cls = [self.cls, self.compare_cls]


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import concurrent.futures
from typing import Any

import numpy as np

from osaft.solutions.base_arf import BaseARF

NDArray = np.ndarray


class ARFData:
    """Container for plotting data for ARF plots

    :param sol: solution
    :param multicore: if `True` the computation is performed on multiple cores
    """

    _instances = 0
    _line_styles = [
        "-",
        "--",
        ":",
        "-.",
        (0, (5, 10)),
        (0, (3, 1, 1, 1)),
        (0, (1, 2)),
        (0, (8, 10)),
        (0, (5, 5)),
        (0, (5, 1)),
        (0, (3, 10, 1, 10)),
        (0, (3, 5, 1, 5)),
        (0, (3, 1, 1, 1)),
        (0, (3, 10, 1, 10, 1, 10)),
        (0, (3, 5, 1, 5, 1, 5)),
        (0, (3, 1, 1, 1, 1, 1)),
    ]

    def __init__(self, sol: BaseARF, multicore: bool = False) -> None:

        self.sol = sol
        self.multicore = multicore

        self._plotting = None
        self._arf = None
        self._norm_arf = None
        self._instance = ARFData._instances
        ARFData._instances += 1

    def compute_arf(self, attr_name: str, values: NDArray) -> None:

        if self.multicore:
            self._arf = self._compute_arf_multi_process(attr_name, values)
        else:
            self._arf = self._compute_arf_single_process(attr_name, values)
        self._plotting = self._arf

    def _compute_arf_single_process(
        self,
        attr_name: str,
        values: NDArray,
    ) -> NDArray:
        arf = []
        for val in values:
            arf.append(
                self._change_attr_and_compute_arf(
                    self.sol,
                    attr_name,
                    val,
                ),
            )
        return np.asarray(arf)

    def _compute_arf_multi_process(
        self,
        attr_name: str,
        values: NDArray,
    ) -> NDArray:
        """Computes the ARF using multiple processes

        The ARF is computes using the `ProcessPoolExecutor` which will
        automatically dispatch the processes.
        """
        sols = [self.sol] * len(values)
        attr_names = [attr_name] * len(values)

        with concurrent.futures.ProcessPoolExecutor() as executor:
            arf = np.fromiter(
                executor.map(
                    self._change_attr_and_compute_arf,
                    sols,
                    attr_names,
                    values,
                ),
                dtype=float,
            )
        return arf

    @staticmethod
    def _change_attr_and_compute_arf(
        sol: BaseARF,
        attr_name: str,
        val: Any,
    ) -> float:
        """Changes the attribute `attr_name` to value `val`

        The attribute is change using `setattr`. Note that the solution has
        to be passed explicit because the method is also used in the
        multiprocessing.

        :attr_name: name of the attribute to be changed
        :attr_name: new value of the attribute
        """
        setattr(sol, attr_name, val)
        return sol.compute_arf()

    def normalize_arf(self, norm: NDArray) -> None:
        self._norm_arf = self._arf / norm
        self._plotting = self._norm_arf

    @property
    def line_style(self) -> str | tuple[int, tuple[int]]:

        L = len(ARFData._line_styles)
        return ARFData._line_styles[self._instance % L]

    @property
    def arf(self) -> NDArray:
        return self._arf

    @property
    def plotting(self) -> NDArray:
        return self._plotting

    @property
    def name(self) -> str:
        return self.sol.name


if __name__ == "__main__":
    pass



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Sequence
from typing import Any

import numpy as np

from osaft import WaveType


class BaseChangingVariable:
    """Type used as attribute of a `TestCase` that is changed automatically.

    To test the observer pattern testing, attributes of classes are
    changed to ensure proper linking. If a test class has an attribute of
    type :class:`BaseChangingVariable` then the attribute of the tested
    class with the name `name` is initially set to `value`. During the
    testing the attribute is changed to a new value using the method
    `change`.

    :param name: name of the attribute in the class to be tested
    :param value: initial value of the attribute
    :param seed: seed for RNG to make testing reproducible
    """

    def __init__(
        self,
        name: str,
        value: Any,
        seed: int,
    ) -> None:
        """Constructor method"""
        self.name = name
        self.value = value

        self.rng = np.random.default_rng(seed)

    @abstractmethod
    def change(self) -> None:
        """Randomly change `value`"""
        pass


class ChangingFromList(BaseChangingVariable):
    """Child of :class:`BaseChangingVariable`.

    The method `change` changes `value` by choosing an item from
    :attr:`list_of_values`.

    :param name: name of the attribute in the class to be tested
    :param value: initial value of the attribute
    :param list_of_values: list of values to choose from
    :param seed: seed for RNG to make testing reproducible
    """

    _longest_list = 0

    def __init__(
        self,
        name: str,
        value: Any,
        list_of_values: Sequence[Any],
        seed: int,
    ) -> None:

        super().__init__(name, value, seed)

        self.list_of_values = list_of_values

    def _set_longest_list(self) -> None:
        if len(self.list_of_values) > self._longest_list:
            ChangingFromList._longest_list = len(self.list_of_values)

    def _reset_non_tested_values(self) -> None:
        # [*list] syntax is needed for not passing a reference but creating an
        # actual new list
        self._non_tested_values = [*self.list_of_values]

    @property
    def list_of_values(self) -> list[any]:
        return self._list_of_values

    @list_of_values.setter
    def list_of_values(self, lst: list[any]) -> None:
        self._list_of_values = lst
        self._set_longest_list()
        self._reset_non_tested_values()

    @property
    def longest_list(self) -> int:
        return ChangingFromList._longest_list

    def change(self) -> None:
        """Randomly change `value`"""
        if len(self._non_tested_values) < 1:
            self._reset_non_tested_values()

        self.value = self.rng.choice(self._non_tested_values)
        self._non_tested_values.remove(self.value)


class ChangingBool(BaseChangingVariable):
    """Child of :class:`BaseChangingVariable`

    Child of :class:`BaseChangingVariable` for the case when value is of type
    `bool`.

    :param name: name of the attribute in the class to be tested
    :param value: initial value of the attribute
    :param seed: seed for RNG to make testing reproducible
    """

    def __init__(self, name: str, value: bool, seed: int) -> None:
        """Constructor method"""
        super().__init__(name, value, seed)

    @abstractmethod
    def change(self) -> None:
        """Randomly change `value`"""
        self.value = self.rng.random() > 0.5


class ChangingNumber(BaseChangingVariable):
    """Child of :class:`BaseChangingVariable`

    Child of :class:`BaseChangingVariable` for the case when value is a number.

    :param name: name of the attribute in the class to be tested
    :param value: initial value of the attribute
    :param seed: seed for RNG to make testing reproducible
    :param low: lowest value `value` is changed to
    :param high: highest value `value` is changed to
    """

    def __init__(
        self,
        name: str,
        value: None | float | int,
        seed: int,
        low: None | float | int,
        high: float | int,
    ) -> None:
        """Constructor method"""
        super().__init__(name, value, seed)
        self.test_inputs(value, low, high)
        self._low = low
        self._high = high

    @staticmethod
    def test_inputs(
        value: None | int | float,
        low: None | int | float,
        high: None | int | float,
    ):
        if low is not None and high is not None:
            assert low <= value <= high, "condition low < value < high not met"
        elif low is not None:
            assert low <= value, "condition low < value < high is not met"
        elif high is not None:
            assert value <= high, "condition low < value < high is not met"

    @property
    def low(self) -> None | int | float:
        return self._low

    @low.setter
    def low(self, value: int | float) -> None:
        if self.high is not None and value > self.high:
            raise ValueError(
                f"low (= {value}) needs to be smaller than high "
                f"(= {self.high})",
            )
        elif value > self.value:
            self._low = value
            self.value = value
            self.change()
        else:
            self._low = value

    @property
    def high(self) -> float | int:
        return self._high

    @high.setter
    def high(self, value: int | float) -> None:
        if self.low is not None and value < self.low:
            raise ValueError(
                f"high (= {value}) needs to be larger than low "
                f"(= {self.low})",
            )
        elif value < self.value:
            self._high = value
            self.value = value
            self.change()
        else:
            self._high = value

    @abstractmethod
    def change(self) -> None:
        """Randomly change `value`"""
        pass


class ChangingFloat(ChangingNumber):
    """Child of :class:`BaseChangingVariable`

    Child of :class:`BaseChangingVariable` for the case when value is a
    `float`.

    :param name: name of the attribute in the class to be tested
    :param value: initial value of the attribute
    :param seed: seed for RNG to make testing reproducible
    :param low: lowest value `value` is changed to
    :param high: highest value `value` is changed to
    """

    def __init__(
        self,
        name: str,
        value: float,
        seed: int,
        low: None | float = None,
        high: None | float = None,
    ) -> None:
        """Constructor method"""
        super().__init__(name, value, seed, low, high)

    def change(self) -> None:
        """Randomly change `value`"""
        if self.low is not None and self.high is not None:
            self.value = self.low + (self.high - self.low) * self.rng.random()
        elif self.low is not None:
            change = self.value * (self.rng.random() - 0.5)
            if self.value + change > self.low:
                self.value += change
            else:
                self.value -= change
        elif self.high is not None:
            change = self.value * (self.rng.random() - 0.5)
            if self.value + change < self.high:
                self.value += change
            else:
                self.value -= change
        else:
            self.value += (self.rng.random() - 0.5) * self.value


class ChangingInt(ChangingNumber):
    """Child of :class:`BaseChangingVariable`

    Child of :class:`BaseChangingVariable` for the case when value is an
    `int`. Default values for `low` and `high` are 0 and 100 respectively. If
    `value` is outside this range the parameters have to be set accordingly.

    :param name: name of the attribute in the class to be tested
    :param value: initial value of the attribute
    :param seed: seed for RNG to make testing reproducible
    :param low: lowest value `value` is changed to
    :param high: highest value `value` is changed to
    """

    def __init__(
        self,
        name: str,
        value: int,
        seed: int,
        low: int = 0,
        high: int = 100,
    ):
        """Constructor method"""
        super().__init__(name, value, seed, low, high)

    def change(self) -> None:
        """Randomly change `value`"""
        self.value = self.rng.integers(low=self.low, high=self.high)


class RandomParameters:
    """Stores and changes a list of parameters commonly used for testing.

    :param default: if ``True`` default parameters are return, else random
       parameters
    """

    def __init__(self, default: bool = True):

        # Create seed
        seed = np.random.randint(1, 10000)
        self.seed = seed
        self.rng = np.random.default_rng(seed)

        # Geometry
        self._R_0 = ChangingFloat("R_0", 1e-6, self.seed, low=0)
        # Frequency
        self._f = ChangingFloat("f", 1e6, self.seed, low=0)
        # Inviscid Fluid
        self._c_f = ChangingFloat("c_f", 1.5e3, self.seed, low=0)
        self._rho_f = ChangingFloat("rho_f", 1e3, self.seed, low=0)
        # Compressible Particle (Inviscid Fluid)
        self._c_s = ChangingFloat("c_s", 2.5e3, self.seed, low=0)
        # Compressible Particle (Viscous Fluid)
        self._eta_s = ChangingFloat("eta_s", 1e-3, self.seed, low=0)
        self._zeta_s = ChangingFloat("zeta_s", 1e-3, self.seed, low=0)
        # Viscous Fluid
        self._eta_f = ChangingFloat(
            "eta_f",
            1e-3,
            self.seed,
            low=1e-5,
            high=1e-2,
        )
        self._zeta_f = ChangingFloat("zeta_f", 1e-3, self.seed, low=0)
        # Viscoelastic Fluid
        self._eta_p = ChangingFloat("eta_p", 1e-3, self.seed, low=0)
        self._zeta_p = ChangingFloat("zeta_p", 1e-3, self.seed, low=0)
        self._lambda_M = ChangingFloat("lambda_M", 1e-3, self.seed, low=0)
        # Rigid Solid
        self._rho_s = ChangingFloat("rho_s", 1.5e3, self.seed, low=0)
        # Elastic solid
        self._E_s = ChangingFloat("E_s", 75e6, self.seed, low=0)
        self._nu_s = ChangingFloat("nu_s", 0.3, self.seed, low=0, high=0.49)
        # Background Field
        self._p_0 = ChangingFloat("p_0", 1e5, self.seed, low=0)
        self._position = ChangingFloat(
            "position",
            np.pi / 4,
            self.seed,
            low=0,
            high=2 * np.pi,
        )
        self._wave_type = ChangingFromList(
            "wave_type",
            WaveType.STANDING,
            WaveType,
            self.seed,
        )

        self._N_max = ChangingInt(
            "N_max",
            2,
            self.seed,
            low=2,
            high=5,
        )

        if not default:
            self.change_all()

        self.list_parameters = self.get_list_parameters()

    # -------------------------------------------------------------------------
    # Methods
    # -------------------------------------------------------------------------

    def change_all(self) -> None:
        for para in self.list_parameters:
            para.change()

    def get_list_parameters(self) -> list:
        """Returns a list of all random parameters"""
        list_parameters = []

        for attr, val in vars(self).items():
            if isinstance(val, BaseChangingVariable):
                list_parameters.append(val)
        return list_parameters

    # -------------------------------------------------------------------------
    # Getters & Setters
    # -------------------------------------------------------------------------

    @property
    def R_0(self) -> float:
        return self._R_0.value

    @R_0.setter
    def R_0(self, value: float) -> None:
        self._R_0.value = value

    @property
    def f(self) -> float:
        return self._f.value

    @f.setter
    def f(self, value: float) -> None:
        self._f.value = value

    @property
    def c_f(self) -> float:
        return self._c_f.value

    @c_f.setter
    def c_f(self, value: float) -> None:
        self._c_f.value = value

    @property
    def rho_f(self) -> float:
        return self._rho_f.value

    @rho_f.setter
    def rho_f(self, value: float) -> None:
        self._rho_f.value = value

    @property
    def c_s(self) -> float:
        return self._c_s.value

    @c_s.setter
    def c_s(self, value: float) -> None:
        self._c_s.value = value

    @property
    def eta_s(self) -> float:
        return self._eta_s.value

    @eta_s.setter
    def eta_s(self, value: float) -> None:
        self._eta_s.value = value

    @property
    def zeta_s(self) -> float:
        return self._zeta_s.value

    @zeta_s.setter
    def zeta_s(self, value: float) -> None:
        self._zeta_s.value = value

    @property
    def rho_s(self) -> float:
        return self._rho_s.value

    @rho_s.setter
    def rho_s(self, value: float) -> None:
        self._rho_s.value = value

    @property
    def eta_f(self) -> float:
        return self._eta_f.value

    @eta_f.setter
    def eta_f(self, value: float) -> None:
        self._eta_f.value = value

    @property
    def zeta_f(self) -> float:
        return self._zeta_f.value

    @zeta_f.setter
    def zeta_f(self, value: float) -> None:
        self._zeta_f.value = value

    @property
    def eta_p(self) -> float:
        return self._eta_p.value

    @eta_p.setter
    def eta_p(self, value: float) -> None:
        self._eta_p.value = value

    @property
    def zeta_p(self) -> float:
        return self._zeta_p.value

    @zeta_p.setter
    def zeta_p(self, value: float) -> None:
        self._zeta_p.value = value

    @property
    def lambda_M(self) -> float:
        return self._lambda_M.value

    @lambda_M.setter
    def lambda_M(self, value: float) -> None:
        self._lambda_M.value = value

    @property
    def E_s(self) -> float:
        return self._E_s.value

    @E_s.setter
    def E_s(self, value: float) -> None:
        self._E_s.value = value

    @property
    def nu_s(self) -> float:
        return self._nu_s.value

    @nu_s.setter
    def nu_s(self, value: float) -> None:
        self._nu_s.value = value

    @property
    def p_0(self) -> float:
        return self._p_0.value

    @p_0.setter
    def p_0(self, value: float) -> None:
        self._p_0.value = value

    @property
    def position(self) -> float:
        return self._position.value

    @position.setter
    def position(self, value: float) -> None:
        self._position.value = value

    @property
    def wave_type(self) -> WaveType:
        return self._wave_type.value

    @wave_type.setter
    def wave_type(self, value: WaveType) -> None:
        self._wave_type.value = value

    @property
    def N_max(self):
        return self._N_max.value

    @N_max.setter
    def N_max(self, value: int):
        self._N_max.value = value


if __name__ == "__main__":
    pass



from __future__ import annotations

from numbers import Number

from osaft import log
from osaft.core.frequency import Frequency
from osaft.core.geometries import Sphere


class BaseFrequencyComposite:
    """Base class for all composite classes with
    :class:`~osaft.core.frequency.Frequency`

    :param frequency: excitation frequency in [Hz]

    This class has two purposes:

        #. Provides an `__init__` methods that handles the `frequency` argument
           correctly to make sure that all components and the composite have
           the same instance of :class:`~osaft.core.frequency.Frequency`.
        #. Provides wrappers for setters and getters of f and omega
    """

    def __init__(self, frequency: int | float | Frequency) -> None:
        """Constructor method"""

        if isinstance(frequency, Number):
            log.debug(
                f"Passed frequency of type {type(frequency)}."
                f"New Frequency object is created.",
            )
            self.frequency = Frequency(frequency)
        elif isinstance(frequency, Frequency):
            log.debug(
                f"Passed frequency of type {type(frequency)}."
                f"No new Frequency object is created",
            )
            self.frequency = frequency
        else:
            raise TypeError(
                f"Expected type of frequency argument to be "
                f"int, float, or Frequency."
                f"got {type(frequency)}.",
            )

        if type(self) is BaseFrequencyComposite:
            log.debug(f"Creating {self}")

    def __repr__(self) -> str:
        """Repr string for debugging"""
        return f"BaseFrequencyComposite(f={self.f})"

    @classmethod
    def input_variables(cls) -> list[str]:
        """Returns all properties that are settable.

        Returns a list of the names of all properties that are settable,
        i.e. all properties that wrap a PassiveVariable.
        """
        list_input_vars = []

        for parent in cls.mro():
            for attr, val in vars(parent).items():
                if isinstance(val, property) and val.fset is not None:
                    list_input_vars.append(attr)
        return list_input_vars

    # -------------------------------------------------------------------------
    # Attribute Wrappers
    # -------------------------------------------------------------------------

    @property
    def f(self) -> float:
        """wrapper for :attr:`osaft.core.frequency.Frequency.f`"""
        return self.frequency.f

    @f.setter
    def f(self, value) -> None:
        self.frequency.f = value

    @property
    def omega(self) -> float:
        """wrapper for :attr:`osaft.core.frequency.Frequency.omega`"""
        return self.frequency.omega


class BaseSphereFrequencyComposite(BaseFrequencyComposite):
    """Composite class with :class:`~osaft.core.frequency.Frequency` and
    :class:`~osaft.core.geometries.Sphere`

    This class has two purposes:

        #. Provides an `__init__` methods that handles the `frequency`
           argument and the `radius` argument correctly to make sure that all
           components and the composite have the same instance of
           :class:`~osaft.core.frequency.Frequency`
           and :class:`~osaft.core.geometries.Sphere`.
        #. Provides wrappers for setters and getters.

    :param frequency: excitation frequency in [Hz]
    :param radius: sphere radius in [m]
    """

    def __init__(
        self,
        frequency: int | float | Frequency,
        radius: int | float | Sphere,
    ) -> None:
        """Constructor method"""

        super().__init__(frequency)

        if isinstance(radius, Number):
            log.debug(
                f"Passed frequency of type {type(radius)}."
                f"New Frequency object is created.",
            )
            self.sphere = Sphere(radius)
        elif isinstance(radius, Sphere):
            log.debug(
                f"Passed sphere of type {type(radius)}."
                f"No new Sphere object is created",
            )
            self.sphere = radius
        else:
            raise TypeError(
                f"Expected type of radius argument to be"
                f"int, float, or Sphere."
                f"got {type(radius)}.",
            )

        if type(self) is BaseSphereFrequencyComposite:
            log.debug(f"Creating {self}")

    def __repr__(self) -> str:
        return f"BaseSphereFrequencyComposite(f={self.f}, " f"r={self.R_0})"

    # -------------------------------------------------------------------------
    # Attribute Wrappers
    # -------------------------------------------------------------------------

    @property
    def R_0(self) -> float:
        """Wrapper for :attr:`osaft.core.geometries.Sphere.R_0`"""
        return self.sphere.R_0

    @R_0.setter
    def R_0(self, value) -> None:
        self.sphere.R_0 = value

    @property
    def area(self) -> float:
        """Wrapper for :attr:`osaft.core.geometries.Sphere.area`"""
        return self.sphere.area

    @property
    def volume(self) -> float:
        """Wrapper for :attr:`osaft.core.geometries.Sphere.volume`"""
        return self.sphere.volume


if __name__ == "__main__":
    pass



import unittest

from osaft.core.backgroundfields import WrongWaveTypeError
from osaft.core.functions import pi, sin
from osaft.tests.basetest_arf import HelperStandingARF, HelperTravelingARF
from osaft.tests.solutions.doinikov1994rigid.setup_test_arf import BaseTestARF


class TestARFLimiting(BaseTestARF):
    def test_fastened_sphere(self) -> None:
        self.assign_parameters()
        self.assertEqual(self.fastened_sphere, self.cls.fastened_sphere)
        self.fastened_sphere = True
        self.assign_parameters()
        self.assertEqual(self.fastened_sphere, self.cls.fastened_sphere)

    def test_NotImplemented_solution(self) -> None:
        self.cls.wave_type = "random"

        self.assertRaises(
            WrongWaveTypeError,
            self.cls.compute_arf,
        )

    # ------------------

    def test_long_wavelength(self) -> None:
        self.long_wavelength = False
        self.assign_parameters()
        self.assertEqual(
            False,
            self.cls.long_wavelength,
        )
        self.long_wavelength = True
        self.assign_parameters()
        self.assertEqual(
            True,
            self.cls.long_wavelength,
        )

    def test_small_boundary_layer(self) -> None:
        self.small_boundary_layer = False
        self.assign_parameters()
        self.assertEqual(
            False,
            self.cls.small_boundary_layer,
        )
        self.small_boundary_layer = True
        self.assign_parameters()
        self.assertEqual(
            True,
            self.cls.small_boundary_layer,
        )

    def test_large_boundary_layer(self) -> None:
        self.large_boundary_layer = False
        self.assign_parameters()
        self.assertEqual(
            False,
            self.cls.large_boundary_layer,
        )
        self.large_boundary_layer = True
        self.assign_parameters()
        self.assertEqual(
            True,
            self.cls.large_boundary_layer,
        )

    # -------------------------------------------------------------------------
    # Properties
    # -------------------------------------------------------------------------

    @property
    def rho_t(self) -> float:
        return self.rho_f / self.rho_s

    @property
    def x_0(self) -> complex:
        return self.cls.field.k_f.real * self.R_0

    @property
    def x(self) -> complex:
        return self.cls.x

    @property
    def x_v(self) -> complex:
        return self.cls.x_v

    @property
    def norm_delta(self) -> complex:
        return self.cls.delta / self.R_0

    # -------------------------------------------------------------------------
    # Test Properties
    # -------------------------------------------------------------------------

    def test_properties(self) -> None:
        properties = [
            "x_0",
            "norm_delta",
        ]
        self._test_properties(properties, threshold=1e-6)


# |x| << 1 << |x_v|


class TestARFLimit1Standing(BaseTestARF, HelperStandingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = False
        self.large_boundary_layer = False
        self.small_boundary_layer = True
        self.assign_parameters()

    def compute_arf(self):
        out = 3 * (1 - self.rho_t) ** 2 / (2 + self.rho_t) ** 2
        out *= self.cls.norm_delta
        out += (5 - 2 * self.rho_t) / (3 * (2 + self.rho_t))

        out *= pi * self.rho_f * self.cls.x_0**3
        out *= self.cls.field.abs_A_squared
        out *= sin(2 * self.position)

        return out


class TestARFLimit1Traveling(BaseTestARF, HelperTravelingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = False
        self.large_boundary_layer = False
        self.small_boundary_layer = True
        self.assign_parameters()

    def compute_arf(self):
        out = 2 + 12 * self.rho_t - 21 * self.rho_t**2 + 7 * self.rho_t**3
        out *= -self.cls.norm_delta / (2 + self.rho_t)

        out += (1 - self.rho_t) ** 2

        out *= 6 * pi * self.rho_f * self.cls.x_0**3 * self.cls.norm_delta
        out *= self.cls.field.abs_A_squared / (2 + self.rho_t) ** 2

        return out


# |x| << |x_v| << 1


class TestARFLimit2Standing(BaseTestARF, HelperStandingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = False
        self.large_boundary_layer = True
        self.small_boundary_layer = False
        self.assign_parameters()

    def compute_arf(self):
        out = 11 * (1 - self.rho_t)
        out /= 5 * self.cls.norm_delta

        out += 2 * self.rho_t - 1

        out *= pi / 3 * self.rho_s * self.cls.x_0**3
        out *= self.cls.field.abs_A_squared
        out *= sin(2 * self.position)

        return out


class TestARFLimit2Traveling(BaseTestARF, HelperTravelingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = False
        self.large_boundary_layer = True
        self.small_boundary_layer = False
        self.assign_parameters()

    def compute_arf(self):
        out = -22 / 15 * pi * self.rho_s * self.cls.x_0**3
        out *= self.cls.field.abs_A_squared

        out *= (1 - self.rho_t) / self.cls.norm_delta

        return out


# |x| << 1 << |x_v|


class TestARFLimit3Standing(BaseTestARF, HelperStandingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = True
        self.large_boundary_layer = False
        self.small_boundary_layer = True
        self.assign_parameters()

    def compute_arf(self):
        out = 5 / 6 + 3 / 4 * self.cls.norm_delta

        out *= pi * self.rho_f * self.cls.x_0**3
        out *= self.cls.field.abs_A_squared
        out *= sin(2 * self.position)

        return out


class TestARFLimit3Traveling(BaseTestARF, HelperTravelingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = True
        self.large_boundary_layer = False
        self.small_boundary_layer = True
        self.assign_parameters()

    def compute_arf(self):
        out = 3 / 2 * pi * self.rho_f * self.cls.x_0**3
        out *= self.cls.field.abs_A_squared

        out *= self.cls.norm_delta * (1 - self.cls.norm_delta)

        return out


# |x| << |x_v| << 1


class TestARFLimit4Standing(BaseTestARF, HelperStandingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = True
        self.large_boundary_layer = True
        self.small_boundary_layer = False
        self.assign_parameters()

    def compute_arf(self):
        out = 1 + 10 / 27 / self.cls.norm_delta

        out *= pi * self.rho_f * self.cls.x_0**3
        out *= self.cls.field.abs_A_squared
        out *= 0.9 * sin(2 * self.position)
        out *= self.cls.norm_delta

        return out


class TestARFLimit4Traveling(BaseTestARF, HelperTravelingARF):
    def setUp(self) -> None:
        super().setUp()
        self.long_wavelength = True
        self.fastened_sphere = True
        self.large_boundary_layer = True
        self.small_boundary_layer = False
        self.assign_parameters()

    def compute_arf(self):
        out = -3 / 2 * pi * self.rho_f * self.cls.x_0**3
        out *= self.cls.field.abs_A_squared

        out *= self.cls.norm_delta**2 * (1 - 6 / 5 / self.cls.norm_delta)

        return out


if __name__ == "__main__":
    unittest.main()



from osaft.core.functions import BesselFunctions as Bf
from osaft.tests.basetest_numeric import BaseTestSolutions
from osaft.tests.solution_factory import SolutionFactory


class BaseKing(BaseTestSolutions):
    def setUp(self) -> None:

        super().setUp()

        self.cls = SolutionFactory().king_1934_base()
        self.list_cls = [self.cls]

    # -------------------------------------------------------------------------
    # Methods
    # -------------------------------------------------------------------------

    @staticmethod
    def phi_n(n: int, arg):
        return -1 / arg**n * Bf.bessely(n, arg)

    @staticmethod
    def psi_n(n: int, arg):
        return 1 / arg**n * Bf.besselj(n, arg)

    def F_n(self, n: int, arg):
        if n == 1:
            out = self.cls.alpha**2 * self.phi_n(n + 1, arg)
            out -= (1 - 1 / self.cls.rho_tilde) * self.phi_n(n, arg)
        else:
            out = self.cls.alpha**2 * self.phi_n(n + 1, arg)
            out -= n * self.phi_n(n, arg)

        return out

    def G_n(self, n: int, arg):
        if n == 1:
            out = self.cls.alpha**2 * self.psi_n(n + 1, arg)
            out -= (1 - 1 / self.cls.rho_tilde) * self.psi_n(n, arg)
        else:
            out = self.cls.alpha**2 * self.psi_n(n + 1, arg)
            out -= n * self.psi_n(n, arg)

        return out

    @property
    def rho_tilde(self):
        return self.cls.rho_s / self.rho_f

    @property
    def alpha(self):
        return self.cls.R_0 * self.cls.k_f

    # -------------------------------------------------------------------------
    # Tests
    # -------------------------------------------------------------------------

    def test_properties(self) -> None:
        properties = ["alpha", "rho_tilde"]
        self._test_properties(properties)



from __future__ import annotations

from abc import ABC

from osaft import log
from osaft.core.backgroundfields import WaveType
from osaft.core.frequency import Frequency
from osaft.core.functions import pi, sin, sqrt
from osaft.core.geometries import Sphere
from osaft.core.helper import StringFormatter as SF
from osaft.core.variable import ActiveVariable, PassiveVariable
from osaft.solutions.basedoinikov1994.arf_limiting import BaseARFLimiting
from osaft.solutions.doinikov1994compressible.scattering import (
    ScatteringField,
)


class ARFLimiting(ScatteringField, BaseARFLimiting, ABC):
    """Base class for Doinikov 1994 (compressible sphere)

    .. note::
        This class implements all attributes and methods to compute the ARF
        in the limiting cases introduces in the article by Doinikov
        :math:`|x| \\ll |x_v| \\ll 1` and :math:`|x| \\ll 1 \\ll |x_v|`.
        This class does not actually implement a method for the ARF. If you
        want to compute the ARF use
        :attr:`~osaft.solutions.doinikov1994compressible.ARF`

    :param f: Frequency [Hz]
    :param R_0: Radius of the sphere [m]
    :param rho_s: Density of the sphere [kg/m^3]
    :param c_s: Speed of sound of in the sphere [m/s]
    :param eta_s: shear viscosity of in the sphere  [Pa s]
    :param zeta_s: bulk viscosity of in the sphere [Pa s]
    :param rho_f: Density of the fluid [kg/m^3]
    :param c_f: Speed of sound of the fluid [m/s]
    :param eta_f: shear viscosity fluid [Pa s]
    :param zeta_f: bulk viscosity fluid [Pa s]
    :param p_0: Pressure amplitude of the field [Pa]
    :param wave_type: Type of wave, travel(l)ing or standing
    :param position: Position within the standing wave field [m]
    :param small_boundary_layer: :math:`x \\ll x_v \\ll 1`
    :param large_boundary_layer: :math`x \\ll 1 \\ll x_v`
    :param N_max: Highest order mode
    :param background_streaming: background streaming contribution
    """

    def __init__(
        self,
        f: Frequency | float | int,
        R_0: Sphere | float | int,
        rho_s: float,
        c_s: float,
        eta_s: float,
        zeta_s: float,
        rho_f: float,
        c_f: float,
        eta_f: float,
        zeta_f: float,
        p_0: float,
        wave_type: WaveType,
        position: None | float,
        small_boundary_layer: bool,
        large_boundary_layer: bool,
        background_streaming: bool,
        N_max: int,
    ) -> None:
        """Constructor method"""

        # init of parent class
        ScatteringField.__init__(
            self,
            f=f,
            R_0=R_0,
            rho_s=rho_s,
            c_s=c_s,
            eta_s=eta_s,
            zeta_s=zeta_s,
            rho_f=rho_f,
            c_f=c_f,
            eta_f=eta_f,
            zeta_f=zeta_f,
            p_0=p_0,
            wave_type=wave_type,
            position=position,
            N_max=N_max,
        )
        BaseARFLimiting.__init__(self)

        # independent variables
        self._small_boundary_layer = PassiveVariable(
            small_boundary_layer,
            "small viscous boundary layer limit",
        )
        self._large_boundary_layer = PassiveVariable(
            large_boundary_layer,
            "large viscous boundary layer limit",
        )

        self._background_streaming = PassiveVariable(
            background_streaming,
            "background streaming contribution",
        )

        # Dependent variables
        self._mu = ActiveVariable(
            self._compute_mu,
            "(density * viscosity) ratio",
        )
        self._kappa_t = ActiveVariable(
            self._compute_kappa_t,
            "compressibility ratio",
        )
        self._eta_t = ActiveVariable(
            self._compute_eta_t,
            "dynamic viscosity ratio",
        )
        self._zeta_t = ActiveVariable(
            self._compute_zeta_t,
            "bulk viscosity ratio",
        )
        self._f1 = ActiveVariable(self._compute_f1, "f1 ARF factor; B.2.1")
        self._f2 = ActiveVariable(self._compute_f2, "f2 ARF factor; B.2.1")
        self._f3 = ActiveVariable(self._compute_f3, "f3 ARF factor; B.2.1")
        self._G = ActiveVariable(self._compute_G, "G ARF factor; eq. 7.9")
        self._S1 = ActiveVariable(self._compute_S1, "S1 ARF factor; eq. 7.10")
        self._S2 = ActiveVariable(self._compute_S2, "S2 ARF factor; eq. 7.11")
        self._S3 = ActiveVariable(self._compute_S3, "S3 ARF factor; eq. 7.12")
        self._f4 = ActiveVariable(self._compute_f4, "f4 ARF factor; B.2.1")
        self._g1 = ActiveVariable(self._compute_g1, "g1 ARF factor; B.2.1")
        self._g2 = ActiveVariable(self._compute_g2, "g2 ARF factor; B.2.1")
        self._g3 = ActiveVariable(self._compute_g3, "g3 ARF factor; B.2.1")
        self._g4 = ActiveVariable(self._compute_g4, "g4 ARF factor; B.2.1")
        self._g5 = ActiveVariable(self._compute_g5, "g5 ARF factor; B.2.1")
        self._g6 = ActiveVariable(self._compute_g6, "g6 ARF factor; B.2.1")
        self._g7 = ActiveVariable(self._compute_g7, "g7 ARF factor; B.2.1")

        # Dependencies
        self._G.is_computed_by(self._S1, self._S2, self._S3)
        self._S1.is_computed_by(
            self._kappa_t,
            self.fluid._rho_f,
            self.scatterer._rho_f,
        )
        self._S2.is_computed_by(self._f1, self._f2, self._f4)
        self._S3.is_computed_by(
            self._f1,
            self._f3,
            self._f4,
            self.fluid._rho_f,
            self.scatterer._rho_f,
            self._kappa_t,
            self._eta_t,
        )
        self._eta_t.is_computed_by(self.scatterer._eta_f, self.fluid._eta_f)
        self._zeta_t.is_computed_by(self.scatterer._zeta_f, self.fluid._zeta_f)
        self._f1.is_computed_by(self._g1, self._g7)
        self._f2.is_computed_by(self._g1, self._g2, self._g5)
        self._f3.is_computed_by(self._g3, self._g6, self._g7)
        self._f4.is_computed_by(self._g4, self._g6, self._g7)
        self._g1.is_computed_by(self._g5, self._g6)
        self._g2.is_computed_by(self._eta_t, self._zeta_t)
        self._g3.is_computed_by(self._eta_t, self._zeta_t)
        self._g4.is_computed_by(self._eta_t, self._zeta_t)
        self._g5.is_computed_by(self._eta_t)
        self._g6.is_computed_by(self._eta_t)
        self._g7.is_computed_by(self._g6)
        self._norm_delta.is_computed_by(self.fluid._delta, self.sphere._R_0)
        self._kappa_t.is_computed_by(
            self.fluid._rho_f,
            self.fluid._c_f,
            self.scatterer._rho_f,
            self.scatterer._c_f,
        )
        self._mu.is_computed_by(
            self.fluid._rho_f,
            self.fluid._eta_f,
            self.scatterer._rho_f,
            self.scatterer._eta_f,
        )

        # logging
        log.debug(repr(self))
        log.info(str(self))

    def __repr__(self):
        return (
            f"Donikov1994Rigid.ARF(frequency={self.f}, radius={self.R_0}, "
            f"rho_s={self.rho_s}, c_s={self.c_s}, "
            f"eta_s={self.eta_s}, zeta_s={self.zeta_s}, "
            f"rho_f={self.rho_f}, c_f={self.c_f}, "
            f"eta_f={self.eta_f}, zeta_f={self.zeta_f}, "
            f"p_0={self.p_0}, position={self.position}, {self.wave_type}, "
            f"small_boundary_layer="
            f"{self.small_boundary_layer}, "
            f"large_boundary_layer="
            f"{self.large_boundary_layer}, "
            f"N_max={self.N_max})"
        )

    def __str__(self):
        out = "Doinikovs's  (1994) model (viscous fluid-rigid sphere) for the"
        out += " ARF with the following properties: \n"
        out += "Limit Cases\n"
        out += SF.get_str_text(
            "Small particle and small delta",
            "Small particle and small delta",
            self.small_boundary_layer,
            None,
        )
        out += SF.get_str_text(
            "Small particle and large delta",
            "Small particle and large delta",
            self.large_boundary_layer,
            None,
        )
        out += "Backgroundfield\n"
        out += SF.get_str_text("Frequency", "f", self.f, "Hz")
        out += SF.get_str_text("Pressure", "p_0", self.p_0, "Pa")
        out += SF.get_str_text("Wavetype", "wave_type", self.wave_type, None)
        out += SF.get_str_text(
            "Position",
            "d",
            self.position,
            "",
        )
        out += SF.get_str_text(
            "Wavelength",
            "lambda",
            self.field.lambda_f,
            "m",
        )

        out += "Fluid\n"
        out += SF.get_str_text(
            "Density",
            "rho_f",
            self.rho_f,
            "kg/m^3",
        )
        out += SF.get_str_text(
            "Sound Speed",
            "c_0",
            self.c_f,
            "m/s",
        )
        out += SF.get_str_text(
            "Compressibility",
            "kappa_f",
            self.kappa_f,
            "1/Pa",
        )
        out += SF.get_str_text(
            "Shear viscosity",
            "eta_f",
            self.eta_f,
            "1/Pa",
        )
        out += SF.get_str_text(
            "Bulk viscosity",
            "zeta_f",
            self.zeta_f,
            "1/Pa",
        )

        out += "Particle\n"
        out += SF.get_str_text(
            "Radius",
            "R_0",
            self.R_0,
            "m",
        )
        out += SF.get_str_text(
            "Density",
            "rho_s",
            self.rho_s,
            "kg/m^3",
        )
        out += SF.get_str_text(
            "Sound Speed",
            "c_0",
            self.c_s,
            "m/s",
        )
        out += SF.get_str_text(
            "Compressibility",
            "kappa_s",
            self.kappa_s,
            "1/Pa",
        )
        out += SF.get_str_text(
            "Shear viscosity",
            "eta_s",
            self.eta_s,
            "1/Pa",
        )
        out += SF.get_str_text(
            "Bulk viscosity",
            "zeta_s",
            self.zeta_s,
            "1/Pa",
        )
        return out

    # -------------------------------------------------------------------------
    # Acoustic radiation force
    # -------------------------------------------------------------------------

    def _arf_small_particle_large_delta(self) -> float:
        """Eq. 7.1 for WaveType.TRAVELLING; Eq. 7.8 for WaveType.STANDING"""
        log.info(
            f"kappa_t << 1 : {self.kappa_t:.3e} << 1\n"
            f"rho_f << rho_s : {self.rho_s:.3e} << {self.rho_s:.3e}\n",
        )

        if self.wave_type == WaveType.TRAVELLING:
            out = 5 - 2 * self.kappa_t - 8 * self.f1

            out *= 1 / 6 * self.norm_delta**2
        else:
            out = self.G * self._standing_wave_position_term()

        out *= pi * self.field.abs_A_squared * self.rho_f * self.x_0**3
        return out

    def _arf_small_particle_small_delta(self) -> float:
        """Eq. 6.1 for WaveType.TRAVELLING"""
        log.info(
            f"kappa_t << 1 : {self.kappa_t:.3e} << 1\n"
            f"mu << 1 : {self.mu:.3e} << 1\n",
        )
        if self.wave_type == WaveType.TRAVELLING:
            out = (self.rho_s - self.rho_f) ** 2
            out /= (2 * self.rho_s + self.rho_f) ** 2 * (1 + self.mu)

            out *= self.x_0**3 * self.norm_delta

            out *= 6 * pi * self.rho_f * self.field.abs_A_squared
        else:
            raise NotImplementedError(
                "This solution is not implemented because Doinikov says "
                "in 6.1.2 of the publication that this solution is "
                "the same as Yosioka already computed.\n If you are "
                "interested in this value us the Yosioka class.",
            )
        return out

    def _standing_wave_position_term(self) -> float:
        return sin(2 * self.k_f.real * self.position)

    # -------------------------------------------------------------------------
    # Dependent variables
    # -------------------------------------------------------------------------

    @property
    def eta_t(self):
        """By fluid normalized dynamic viscosity"""
        return self._eta_t.value

    def _compute_eta_t(self):
        return self.scatterer.eta_f / self.fluid.eta_f

    @property
    def zeta_t(self):
        """By fluid normalized bulk viscosity"""
        return self._zeta_t.value

    def _compute_zeta_t(self):
        return self.scatterer.zeta_f / self.fluid.zeta_f

    @property
    def f1(self):
        """f1 factor of Appendix B"""
        return self._f1.value

    def _compute_f1(self):
        return self.g1 / self.g7

    @property
    def f2(self):
        """f2 factor of Appendix B"""
        return self._f2.value

    def _compute_f2(self):
        if self.g1 == 0:  # pragma: no cover
            return 0
        return self.g2 * self.g5 / self.g1 / 2

    @property
    def f3(self):
        """f3 factor of Appendix B"""
        return self._f3.value

    def _compute_f3(self):
        return self.g3 * self.g6 / self.g7 / 10

    @property
    def f4(self):
        """f4 factor of Appendix B"""
        return self._f4.value

    def _compute_f4(self):
        return self.g4 * self.g6 / self.g7 / 2

    @property
    def G(self):
        """G factor Eq (7.9)"""
        return self._G.value

    def _compute_G(self):
        return (self.S1 + self.S2 + self.S3) / 3

    @property
    def S1(self):
        """S_1 factor Eq (7.10)"""
        return self._S1.value

    def _compute_S1(self):
        return 1 - self.kappa_t + 0.9 * (1 - self.rho_s / self.rho_f)

    @property
    def S2(self):
        """S_2 factor Eq (7.11)"""
        return self._S2.value

    def _compute_S2(self):
        out = 2 * self.f2 - 1
        out *= 2 * self.f1

        out -= 4 * self.f4
        return out

    @property
    def S3(self):
        """S_3 factor Eq (7.12)"""
        return self._S3.value

    def _compute_S3(self):
        out = 3 - 1 / self.eta_t
        out *= -120 * (self.f3 - self.f4)

        out += 50 * self.f1 + 43 - 10 * self.kappa_t

        out *= self.rho_s - self.rho_f
        out /= 30 * self.rho_f * (3 + 2 / self.eta_t)

        return out

    @property
    def g1(self):
        """g1 factor of Appendix B"""
        return self._g1.value

    def _compute_g1(self):
        return self.g5 * self.g6

    @property
    def g2(self):
        """g2 factor of Appendix B"""
        return self._g2.value

    def _compute_g2(self):
        out = 209 + 148 * self.eta_t + 48 / self.eta_t
        out *= -1 / 9 / self.zeta_t

        out += 19 + 38 * self.eta_t - 12 / self.eta_t
        return out

    @property
    def g3(self):
        """g3 factor of Appendix B"""
        return self._g3.value

    def _compute_g3(self):
        out = 25 + 370 * self.eta_t - 80 / self.eta_t
        out *= 1 / 9 / self.zeta_t

        out -= 12 + 19 * self.eta_t + 4 / self.eta_t
        return out

    @property
    def g4(self):
        """g4 factor of Appendix B"""
        return self._g4.value

    def _compute_g4(self):
        out = 5 + 74 * self.eta_t - 16 / self.eta_t
        out *= 1 / 9 / self.zeta_t

        out -= 3 + 4 / self.eta_t
        return out

    @property
    def g5(self):
        """g5 factor of Appendix B"""
        return self._g5.value

    def _compute_g5(self):
        out = 1 - self.eta_t
        out *= 19 + 16 / self.eta_t
        return out

    @property
    def g6(self):
        """g6 factor of Appendix B"""
        return self._g6.value

    def _compute_g6(self):
        return 89 + 48 / self.eta_t + 38 * self.eta_t

    @property
    def g7(self):
        """g7 factor of Appendix B"""
        return self._g7.value

    def _compute_g7(self):
        return self.g6**2

    @property
    def mu(self) -> float:
        """(density x viscosity) ratio
        :math:`\\tilde{\\mu}=\\frac{\\rho_f\\eta_f}{\\rho_s\\eta_s}`
        """
        return self._mu.value

    def _compute_mu(self):
        out = self.rho_f * self.eta_f
        out /= self.rho_s * self.eta_s
        return sqrt(out)

    @property
    def kappa_t(self) -> float:
        """compressibility ratio
        :math:`\\tilde{\\kappa}=\\frac{\\kappa_s}{\\kappa_f}`
        """
        return self._kappa_t.value

    def _compute_kappa_t(self):
        return self.kappa_s / self.kappa_f


if __name__ == "__main__":
    pass



import unittest

import numpy as np

from osaft import WaveType
from osaft.solutions.base_solution import BaseSolution
from osaft.tests.basetest_numeric import BaseTestSolutions
from osaft.tests.testing_parameters import (
    ChangingBool,
    ChangingFloat,
    ChangingFromList,
    ChangingInt,
)


class DummyClass(BaseSolution):
    def __init__(self) -> None:
        super().__init__("not_empty")

        self._wave_type = WaveType.STANDING

    @property
    def wave_type(self):
        return self._wave_type

    @wave_type.setter
    def wave_type(self, value):
        self._wave_type = value

    @property
    def f1(self) -> float:
        return 42.0

    @property
    def f2(self) -> float:
        return -42.0

    def func_1(self, n: int) -> complex:
        return 1 + 1j * (-1.0) ** n

    def func_2(self, n: int, arg: complex) -> complex:
        return 1 + arg * (-1.0) ** n

    def func_3(self, n: int, arg1: complex, arg2: complex) -> complex:
        return 1 + arg2 * (-1.0) ** n - arg2

    def input_variables(self) -> list:
        return []


class TestBaseTest(BaseTestSolutions):
    def setUp(self) -> None:
        super().setUp()
        self.cls = DummyClass()

        self.list_cls = [self.cls]

        self.rng = np.random.default_rng(self.parameters.seed)

    @property
    def f1(self) -> float:
        return 42.0

    @property
    def f2(self) -> float:
        return -42.0

    def func_1(self, n: int) -> complex:
        return 1 + 1j * (-1.0) ** n

    def func_2(self, n: int, arg: complex) -> complex:
        return 1 + arg * (-1.0) ** n

    def func_3(self, n: int, arg1: complex, arg2: complex) -> complex:
        return 1 + arg2 * (-1.0) ** n - arg2

    def test_properties(self) -> None:
        properties = ["f1", "f2"]
        self._test_properties(properties)

    def test_methods_n(self) -> None:
        methods = {}
        methods["func_1"] = None

        arg1 = self.rng.random() * (1j - 1)
        methods["func_2"] = arg1

        arg2 = self.rng.random() * (-1j + 1)
        methods["func_3"] = (arg1, arg2)

        self._test_methods_n(methods)

    def test_longest_list(self) -> None:
        for val in self.parameters.list_parameters:
            if isinstance(val, ChangingFromList):
                self.assertEqual(self.n_runs, val.longest_list)

    def test_longest_list_ValueError(self) -> None:
        for val in self.parameters.list_parameters:
            if isinstance(val, ChangingFromList):
                self.assertRaises(
                    ValueError,
                    self._set_n_runs,
                    val.longest_list - 1,
                )

    def _set_n_runs(self, val):
        self.n_runs = val

    def test_do_testing_zero(self):

        small_value_func_1 = lambda: 1e-20
        small_value_func_2 = lambda: 1e-18

        # Check if AssertionError without zero argument
        do_testing = lambda: (
            self.do_testing(
                func_1=small_value_func_1,
                func_2=small_value_func_2,
            )
        )
        self.assertRaises(AssertionError, do_testing)

        # Check if no error with
        self.do_testing(
            func_1=small_value_func_1,
            func_2=small_value_func_2,
            zero=1e-19,
        )


class TestChangingFromList(unittest.TestCase):
    def setUp(self) -> None:

        self.n_runs = 10
        self.seed = 1

        self.initial_value = "dog"
        self.list = ["dog", "cat", "mouse"]

        # [*list] syntax is needed for not passing a reference but creating an
        # actual new list
        self.cls = ChangingFromList(
            "test_list",
            self.initial_value,
            [*self.list],
            self.seed,
        )

    def test_change(self):
        self.assertIn(self.cls.value, self.list)
        for _ in range(self.n_runs):
            self.cls.change()
            self.assertIn(self.cls.value, self.list)

    def test_if_changing(self):
        self.cls.value = "dog"
        while self.cls.value == "dog":
            self.cls.change()

    def test_class_variable(self):
        A = ChangingFromList("A", "a", ["a", "b"], 0)
        B = ChangingFromList("B", "b", ["a", "b", "c"], 0)

        self.assertEqual(3, A.longest_list)
        self.assertEqual(3, B.longest_list)


class TestChangingBool(unittest.TestCase):
    def setUp(self) -> None:

        self.n_runs = 10
        self.seed = 1

        self.cls = ChangingBool("test_bool", True, self.seed)

    def test_change(self):
        self.cls.change()
        for _ in range(self.n_runs):
            self.assertIn(self.cls.value, [True, False])

    def test_value_changes(self):
        self.cls.value = True
        while self.cls.value:
            self.cls.change()


class BaseTestChangingNumber:
    class TestChangingNumber(unittest.TestCase):
        def setUp(self) -> None:

            self.n_runs = 10
            self.seed = 1

            self.cls = None
            self.initial_value = None
            self.low = None
            self.high = None
            self.seed = None

        def test_set_low_value_error(self):
            # Test Value Error
            self.cls.high = self.high
            with self.assertRaises(ValueError):
                self.cls.low = self.high + 1

        def test_set_low_reset_value(self):
            # Reset Value if value > low
            self.cls.value = self.low - 1
            self.cls.low = self.low
            self.assertTrue(self.cls.value > self.low)

        def test_set_high_value_error(self):
            # Test Value Error
            self.cls.low = self.low
            with self.assertRaises(ValueError):
                self.cls.high = self.low - 1

        def test_set_high_reset_value(self):
            # Reset Value if value > low
            self.cls.value = self.high + 1
            self.cls.high = self.high
            self.assertTrue(self.cls.value < self.high)

        def test_change(self):
            self.assertEqual(self.cls.value, self.initial_value)
            self.cls.change()
            self.assertNotEqual(self.cls.value, self.initial_value)

        def test_low(self):
            self.cls.low = self.low
            for _ in range(self.n_runs):
                self.cls.change()
                self.assertTrue(self.cls.value >= self.low)

        def test_high(self):
            self.cls.high = self.high
            for _ in range(self.n_runs):
                self.cls.change()
                self.assertTrue(self.cls.value <= self.high)

        def test_low_high(self):
            self.cls.low = self.low
            self.cls.high = self.high
            for _ in range(self.n_runs):
                self.cls.change()
                self.assertTrue(self.cls.value >= self.low)
                self.assertTrue(self.cls.value <= self.high)

        def test_low_high_negative(self):
            self.low = -2.7
            self.high = -0.5
            self.test_low_high()


class TestChangingFloat(BaseTestChangingNumber.TestChangingNumber):
    def setUp(self) -> None:
        super().setUp()

        self.initial_value = 2.5
        self.low = 0.5
        self.high = 2.7
        self.seed = 1

        self.cls = ChangingFloat(
            "test_float",
            self.initial_value,
            self.seed,
        )


class TestChangingInt(BaseTestChangingNumber.TestChangingNumber):
    def setUp(self) -> None:
        super().setUp()

        self.initial_value = 4
        self.low = 1
        self.high = 12
        self.seed = 1

        self.cls = ChangingInt(
            "test_int",
            self.initial_value,
            self.seed,
        )


if __name__ == "__main__":
    unittest.main()



from abc import ABC, abstractmethod


class BaseARF(ABC):
    """Base class for the Acoustic Radiation Force that defines the common
    interface.
    """

    @abstractmethod
    def compute_arf(self) -> float:
        """Returns the value for the ARF in Newton [N].
        This method must be implemented by every theory to have a common
        interface for other modules.
        """
        pass


if __name__ == "__main__":
    pass
